



#include "SoundInputPort.h"

#include "EgOSUtils.h"
#include <math.h>

#define __err( msg )			EgOSUtils::ShowMsg( msg );

#if EG_MAC
#define __blockmove( srce, dest, num )  ::BlockMove( (char*) (srce), (char*) (dest), (num) )
#endif



#if EG_WIN
#define __blockmove( srce, dest, num )  ::MoveMemory( (dest), (srce), (num) )

#define __getBufInfo																		\
						if ( mHdr1.dwFlags & WHDR_DONE ) {									\
							mHdr1.dwBytesRecorded = 0;										\
							waveInAddBuffer( mWaveDevice, &mHdr1, sizeof( mHdr1 ) );		\
						}																	\
						if ( mHdr2.dwFlags & WHDR_DONE  ) {									\
							mHdr2.dwBytesRecorded = 0;										\
							waveInAddBuffer( mWaveDevice, &mHdr2, sizeof( mHdr2 ) );		\
						}																	\
																							\
						/* See where the most recent recorded data ends */					\
						MMTIME pos;		pos.wType = TIME_BYTES;								\
						waveInGetPosition( mWaveDevice, &pos, sizeof( pos ) );				\
																							\
						/* Calc the starting element number		 */							\
						long n = mHdr1.dwBufferLength / sizeof( short );					\
						long i, start = ( pos.u.cb / sizeof( short ) ) % n - 900 - inN;		\


						
#endif


SoundInput::SoundInput() :
	mAnalyzer() {
	
	mIsRecording			= false;
	mParams.mNumSampleSkip	= 0;
	mParams.mSamples		= NULL;
	mParams.mNumSamples		= 0;
	
	DimBufSize( 16 );
}


SoundInput::~SoundInput() {

	StopRecording();
}



void SoundInput::DimBufSize( long inNumSamples ) {
	long i, need;
	bool wasRecording = mIsRecording;
			
	if ( inNumSamples != mParams.mNumSamples ) {

		// times 2 b/c we wrap our data
		need = 2 * sizeof( short ) * inNumSamples;
		
		#if EG_WIN
		if ( need < 0x4000 )
			need = 0x4000;
		#endif

		StopRecording();
		mParams.mSamples		= (short*) mParamBuf.Dim( need + sizeof( short ) * inNumSamples );
		mParams.mNumSamples		= inNumSamples;
		mParams.mBufEnd 		= 0;
		mParams.mInUse			= 0;
		
		#if EG_WIN
		mHdr1.lpData			= (char*) mParams.mSamples;
		mHdr1.dwBufferLength	= need;
		mHdr1.dwBytesRecorded	= 0;
		mHdr1.dwFlags			= 0;
		mHdr1.dwUser			= 0;
		mHdr1.dwLoops			= 0;

		mHdr2 = mHdr1;
		#endif
		
		for ( i = 0; i < inNumSamples; i++ )
			mParams.mSamples[ i ] = 0;
	}
			
	
	if ( wasRecording )
		StartRecording();
}


bool SoundInput::GetSamples( short outSamples[], long inN ) {
	
	DimBufSize( inN );
	
	#if EG_MAC
	long start, n;
	
	// Signal to our interrupt handler that we're using the buffer
	mParams.mInUse = 1;
	
	// We use a circular buffer, so to get the sequnce in order, paste the last half after the first half
	start = mParams.mBufEnd;	
	n = mParams.mNumSamples - start;
	__blockmove( &mParams.mSamples[ start ], outSamples, sizeof( short ) * n );		
	__blockmove( &mParams.mSamples[ 0 ], &outSamples[ n ], sizeof( short ) * ( inN - n ) );
	
	// Signal we're not using the buffer anymore
	mParams.mInUse = 0;
	#endif
	
	#if EG_WIN
	__getBufInfo
		
	for ( i = 0; i < - start && i < inN; i++ )
		outSamples[ i ] = mParams.mSamples[ n + start + i ];
	for ( ; i < inN; i++ )
		outSamples[ i ] = mParams.mSamples[     start + i ];
	#endif
	
	return true;
}



bool SoundInput::GetSamples( float outSamples[], long inN ) {

	
	DimBufSize( inN );

	#if EG_MAC
	long i, n;

	// Signal to our interrupt handler that we're using the buffer
	mParams.mInUse = 1;
	
	// We use a circular buffer, so to get the sequnce in order, paste the last half after the first half
	n = mParams.mNumSamples - mParams.mBufEnd;
	for ( i = 0; i < n; i++ )
		outSamples[ i ] = mParams.mSamples[ i + mParams.mBufEnd ];
	for ( i = n; i < inN; i++ )
		outSamples[ i ] = mParams.mSamples[ i - n ];
		
	// Signal we're not using the buffer anymore
	mParams.mInUse = 0;
	#endif
	
	#if EG_WIN
	__getBufInfo
	
	// We use a circular buffer, so mind it...
	for ( i = 0; i < - start && i < inN; i++ )
		outSamples[ i ] = mParams.mSamples[ n + start + i ];
	for ( ; i < inN; i++ )
		outSamples[ i ] = mParams.mSamples[     start + i ];
	#endif

	return true;
}



bool SoundInput::GetSpectrum( long inBins, float inBinRange, float outFT[] ) {

	
	DimBufSize( 400 );
	
	#if EG_MAC
	long i;
	
	// Signal to our interrupt handler that we're using the buffer
	mParams.mInUse = 1;
		
	// We use a circular buffer, so to get the sequnce in order, paste the last half after the first half
	i = mParams.mBufEnd;	
	__blockmove( mParams.mSamples, &mParams.mSamples[ mParams.mNumSamples ], sizeof( short ) * i );
	#endif
	
	#if EG_WIN
	long inN = mParams.mNumSamples;
	
	__getBufInfo
	
	// Don't bother moving samples if we're not wrapping
	if ( start + inN < 0 ) 
		start += n;
		
	// Is our data wrapped around the edge of our circular buffer?
	if ( start < 0 ) {

		// Paste the last portion after the first portion (that's why we allocated extra)
		for ( i = 0; i < start + inN; i++ )
			mParams.mSamples[ n + i ] = mParams.mSamples[ i ];

		// Make start contain a valid starting index
		start += n;
		
	}
	i = start;
	#endif
	
	// Turn a sequence of samples into a freq spectrum
	mAnalyzer.Transform( &mParams.mSamples[ i ], mParams.mNumSamples, inBins, inBinRange, outFT );
	
	// Signal we're not using the buffer anymore
	mParams.mInUse = 0;
		
	return true;
}




#if EG_MAC
static OSErr	SetupDevice (long inputDevice,
							short *numChannels,
							short *sampleSize,
							Fixed *sampleRate,
							OSType *compression,
							long *devBuffer) {

	OSErr				err;
	Fixed				gain = 0x00008000;
	short				on = 1;

	#if EG_MAC
	err = SPBSetDeviceInfo (inputDevice, siSampleRate, (Ptr) sampleRate);
	if (err != noErr)
		__err( "Couldn't set sample rate" );

	err = SPBSetDeviceInfo (inputDevice, siSampleSize, (Ptr) sampleSize);
	if (err != noErr)
		__err( "Couldn't set sample size");

	err = SPBSetDeviceInfo (inputDevice, siTwosComplementOnOff, (Ptr) &on);
	if (err != noErr)
		__err( "Couldn't set twos complement");

	err = SPBSetDeviceInfo (inputDevice, siNumberChannels, (Ptr) numChannels);
	if (err != noErr)
		__err( "Couldn't set number of channels");

	err = SPBSetDeviceInfo (inputDevice, siCompressionType, (Ptr) compression);
	if (err != noErr)
		__err( "Couldn't set compression type");

	//turn on continuous recording to "warm up" the input device
	err = SPBSetDeviceInfo (inputDevice, siContinuous, (Ptr) &on);
	if (err != noErr)
		__err( "Couldn't turn on continuous recording");

	//turn on Automatic Gain Control
	err = SPBSetDeviceInfo (inputDevice, siAGCOnOff, (Ptr) &on);
	if (err != noErr) {
		//If AGC isn't available, just turn it all the way down to avoid over driving
		err = SPBSetDeviceInfo (inputDevice, siInputGain, (Ptr) &gain);
		if (err != noErr)
			__err( "Couldn't get siInputGain");
	}

	//check to see what we really got
	err = SPBGetDeviceInfo (inputDevice, siSampleRate, (Ptr) sampleRate);
	if (err != noErr)
		__err( "Couldn't get sample rate");

	err = SPBGetDeviceInfo (inputDevice, siSampleSize, (Ptr) sampleSize);
	if (err != noErr)
		__err( "Couldn't get sample size");

	err = SPBGetDeviceInfo (inputDevice, siNumberChannels, (Ptr) numChannels);
	if (err != noErr)
		__err( "Couldn't get number of channels");

	err = SPBGetDeviceInfo (inputDevice, siDeviceBufferInfo, (Ptr) devBuffer);
	if (err != noErr)
		__err( "Couldn't get number of channels");

	err = SPBGetDeviceInfo (inputDevice, siCompressionType, (Ptr) compression);
	if (err != noErr)
		__err( "Couldn't get compression type");
	#endif
	
	return err;
}
#endif



OSErr SoundInput::StartRecording () {
	OSErr				err;

	if ( ! mIsRecording ) {

		#if EG_WIN
		mParams.recordRec.wFormatTag		= 1;
		mParams.recordRec.nChannels			= 1;
		mParams.recordRec.nSamplesPerSec	= 11025;
		mParams.recordRec.wBitsPerSample	= 16;
		mParams.recordRec.nAvgBytesPerSec	= mParams.recordRec.nSamplesPerSec * mParams.recordRec.wBitsPerSample;
		mParams.recordRec.nBlockAlign		= mParams.recordRec.nChannels * mParams.recordRec.wBitsPerSample / 8;
		mParams.recordRec.cbSize			= 0;
		mWaveDevice							= 0;

		if ( waveInOpen( &mWaveDevice, 0, &mParams.recordRec, 0, 0, 0) == MMSYSERR_NOERROR ) {
			
			if (	waveInPrepareHeader( mWaveDevice, &mHdr1, sizeof( mHdr1 ) ) == MMSYSERR_NOERROR &&
					waveInPrepareHeader( mWaveDevice, &mHdr2, sizeof( mHdr2 ) ) == MMSYSERR_NOERROR ) {
					
				if ( 	waveInAddBuffer( mWaveDevice, &mHdr1, sizeof( mHdr1 ) )	== MMSYSERR_NOERROR &&
						waveInAddBuffer( mWaveDevice, &mHdr2, sizeof( mHdr2 ) ) == MMSYSERR_NOERROR ) {
					
					// Start the recording
					if ( waveInStart( mWaveDevice ) == MMSYSERR_NOERROR ) {
						err = 0;
						mIsRecording = true;
					}
					
				}
				
			}
						
			// Cleanup if we opened the device and then had probs
			if ( ! mIsRecording )
				waveInStop( mWaveDevice );
		}
		#endif
		
		#if EG_MAC
		err = SPBOpenDevice (nil, siWritePermission, &mParams.soundRefNum);
		if (err != noErr) {
			__err( "Couldn't open the device");
		}

		mParams.numChannels	= 1;
		mParams.sampleSize	= 16;
		mParams.sampleRate	= rate44khz;
		mParams.compression	= 'NONE';
		err = SetupDevice (mParams.soundRefNum, &mParams.numChannels,
												&mParams.sampleSize,
												&mParams.sampleRate,
												&mParams.compression,
												&mParams.devBuffer);



		if (err == noErr) {
		/*	IOCompletionUPP 	MyPB0WriteCompUPP,
								MyPB1WriteCompUPP;*/
			//SICompletionUPP 	MyRecCompUPP;
			SIInterruptUPP		MyIntCompUPP;

			//mParams->sanitycheck = 'SANE';
			//myVars->whichBuffer = 0;
			mParams.myA5 = SetCurrentA5 ();

			//set up the record parameters
			//MyRecCompUPP = NewSICompletionProc (MyRecComp);
			MyIntCompUPP = NewSIInterruptProc (MyRecInterrupt);
			mParams.recordRec.inRefNum = mParams.soundRefNum;
			mParams.recordRec.count = 0;
			mParams.recordRec.milliseconds = 0;
			mParams.recordRec.bufferLength = 0 ;//buffSize;
			mParams.recordRec.bufferPtr = NULL; //myVars->recBuffer;
			mParams.recordRec.completionRoutine = NULL; // MyRecCompUPP;
			mParams.recordRec.interruptRoutine = MyIntCompUPP;
			mParams.recordRec.userLong = (long) &mParams;
			mParams.recordRec.error = 0;
			mParams.recordRec.unused1 = 0;
			mParams.mInUse = false;

		}


		//mParams->totalBytes = 0;
		err = SPBRecord (&mParams.recordRec, true);
		if (err != noErr) {
			__err( "SPBRecord failed");
		}
		
		if ( err == noErr )
			mIsRecording = true;

		#endif
	}
	
	return err;
}




void SoundInput::StopRecording () {
	
	if ( mIsRecording ) {
	
		#if EG_MAC
		OSErr err;
		err = SPBStopRecording (mParams.soundRefNum);
		if (err != noErr)
			__err( "SPBStopRecording failed");

		err = SPBCloseDevice (mParams.soundRefNum);
		if (err != noErr)
			__err( "SPBCloseDevice failed");
		#endif
			
			
		#if EG_WIN
		waveInReset( mWaveDevice );
		waveInUnprepareHeader( mWaveDevice, &mHdr1, sizeof( mHdr1 ) );
		waveInUnprepareHeader( mWaveDevice, &mHdr2, sizeof( mHdr2 ) );
		if ( waveInStop( mWaveDevice ) != MMSYSERR_NOERROR ) {
			__err( "waveInStop() failed"); }
		else if ( waveInClose( mWaveDevice ) != MMSYSERR_NOERROR ) {
			__err( "waveInClose() failed");
		}
		#endif
		
		mIsRecording = false;
	}
	
}



#if EG_MAC
pascal void		MyRecInterrupt( SPB* inPB, short* buf, long peak, long size ) {

	//SPBPtr inPB:__a0;
	#pragma unused( peak )
	
	SoundInputRec*			recPtr;
	int i;
	long oldA5, offset, skip;

	recPtr = (SoundInputRec*)inPB->userLong;

	oldA5 = SetA5 (recPtr->myA5);
	
	// We can only write to the buf if its not in use
	if ( ! recPtr -> mInUse ) {
	
		offset = recPtr -> mBufEnd;
		skip = 1 + recPtr -> mNumSampleSkip;
		
		for ( i = 0; i < size; i += skip ) {
			recPtr -> mSamples[ offset ] = buf[ i ];
			offset = ( offset + 1 ) % recPtr -> mNumSamples;
		}
		
		recPtr -> mBufEnd = offset;
	}

	oldA5 = SetA5 (oldA5);
}
#endif

