
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "entheo_api.h"
#include <Events.h>

#include "CEgFileSpec.h"
#include "WhiteCap.h"
#include "EgOSUtils.h"


int Plugin_Init();
void Plugin_Draw(VisInfoBlock *block);
void Plugin_Depth(Rect *rBounds);
void Plugin_Term();
int Plugin_Key(EventRecord *event);
int Plugin_Mouse(EventRecord *event);
int Plugin_Control(int control, int data);

vis_plug_header plug_entry =
{
	/*signature*/				kVIS_PluginCreator,
	/*apiVersion*/				kVIS_API_Version,
	/*name*/					"WhiteCap",
	/*version*/					"3.4",
	/*copyright*/				"2000, 55Ware",
	/*description*/				"Visual Plugin",
	/*url*/						"http://55ware.audiogalaxy.com",
	/*plug_init*/				Plugin_Control
};


static WhiteCap*		gWC = NULL;
static int				gEnabled = 0;
static Rect				gRect = { -1, -1, -2, -2 };






int Plugin_Control(int control, int data)
{
	switch (control)
	{
		case VIS_CONTROL_INIT:
			return Plugin_Init();
		case VIS_CONTROL_TERM:
			Plugin_Term();
			break;
		case VIS_CONTROL_ISENABLED:
			return gEnabled;
		case VIS_CONTORL_ENABLE:
			gEnabled = data;
			break;
		case VIS_CONTROL_MOUSE:
			return Plugin_Mouse((EventRecord*)data);
		case VIS_CONTROL_KEY:
			return Plugin_Key((EventRecord*)data);
		case VIS_CONTORL_DRAW:
			Plugin_Draw((VisInfoBlock*)data);
			break;
	}
	return 0;
}


int Plugin_Init()
{
	EgOSUtils::Initialize();
	
	ScreenDevice::sMinDepth = 32;
		
	CEgFileSpec folder;
	folder.AssignFolder( "WhiteCap Configs" );
	gWC = new WhiteCap( folder );
	
	return 0;
}




void Plugin_Term() {

	if ( gWC ) {
		delete gWC;
		gWC = NULL;
	}
	
	EgOSUtils::Shutdown();
}


#define SKIP_BINS 0

void Plugin_Draw(VisInfoBlock *inVis ) {
	Rect draw = *inVis -> rBounds;
	long bytesPerPix, offset;

	// Don't draw on the border unless we're in borerless mode
	if ( ! gWC -> BorderlessWindow() ) {
		draw.left += 6;
		draw.top += 6;
		draw.right -= 7;
		draw.bottom -= 7;
	}
	
	bytesPerPix = inVis -> rDepth / 8;
	offset = ( draw.left - inVis -> rBounds -> left ) * bytesPerPix + ( draw.top - inVis -> rBounds -> top ) * inVis -> rowBytes;
	
	// Determine if we've resized
	if ( ! ::EqualRect( &gRect, inVis -> rBounds ) ) {
		gRect = *inVis -> rBounds;
		gWC -> SetPort( inVis -> port, draw, false );	
	}

	// Tell WC where to draw the next frame	
	gWC -> RedirectFrame( (char*) inVis -> baseAddr + offset, inVis -> rowBytes, inVis -> rDepth, draw.right - draw.left, draw.bottom - draw.top );
	
	int i, j, bin;
	unsigned long status = 0;
	unsigned short fftNum;
	float* samp = *inVis -> spectrum;
	float* sample = gWC -> mSample, binVal;
	long stepsPerBin = gWC -> mStepsPerBin, numBins = gWC -> mNumSampleBins;
	
	
	fftNum = FFTSIZE - SKIP_BINS;
	if ( fftNum < 0 )
		fftNum = 0; 
	
	// Restrict case when we have too many WC bins
	if ( numBins > fftNum )
		numBins = fftNum;
	
	// Limit number of bins if there's not enough mp3 player sample to go around
	else if ( numBins * stepsPerBin > fftNum )
		numBins = fftNum / stepsPerBin;
	
	
	bin = SKIP_BINS;
	for ( i = 0; i < numBins; i++ ) {
		binVal = 0;
		for ( j = 0; j < stepsPerBin; j++, bin++ ) 
			binVal += samp[ bin ];

		//sample[ i ] = log( .005 * sqrt( binVal ) + 1.6 ) - log( 1.6 );
		sample[ i ] = sqrt( 25 + .00042 * binVal * (0.85 + .051 * i) ) - sqrt( 25 );
		
	}
	
	sample[ 0 ] *= 0.35;
	sample[ 1 ] *= 0.65;
	
	// If we didn't have enough bins to fill the WC sample, zero out the rest
	for ( i = numBins; i < gWC -> mNumSampleBins; i++ )
		sample[ i ] = 0;

	// Store the newest sample and chuck and old samples
	gWC -> RecordSample( EgOSUtils::CurTimeMS(), 205.5 );


	// Update the screen, baby.  Let's shag!
	gWC -> Draw();
		



	/*

	float* index = inVis -> spectrum;
	
	{
		RGBForeColor(&spectrumColor);
		for (i=1; (i<FFTSIZE/2)&&(i*2<280); i++)
		{
			height1 = 1+(int)(*key)[i];
			MoveTo(i*2+17, 129+height1);
	    	LineTo(i*2+17, 129-height1);
	    }
	}
	if ((key=block->pcm) == NULL)
	{
		PenSize(1,1);
		return;
	}
	if (thePrefs.mDigital)
	{
		RGBForeColor(&digitalColor);

		height1 = (*key)[2] * 0.5;
		height1&=0xf0;
		height1>>=1;
		MoveTo(17, 200-height1);
		for (int i=2; (i<FFTSIZE)&&(i*2<280); i++)
		{
			height1 = ((*key)[i*2] * 0.5);
			height1&=0xf0;
			height1>>=1;
	    	LineTo(i*2+17, 200-height1);
	    }
	}
	PenSize(1,1);
	if (thePrefs.mOscillo)
	{
		RGBForeColor(&oscilloColor);
		
		//height1 = 1+(int)(*key)[2];
		//MoveTo(2+17, 129-height1);
		for (i=2; (i<FFTSIZE/2)&&(i*2<280); i++)
		{
			height1 = 1+(int)(*key)[i];
			MoveTo(i*2+16, 129-height1);
	    	LineTo(i*2+17, 129-height1);
	    }
	}
	*/
	
	if ( inVis -> info ) {
		//char *title = &block->info->mTrackTitle[0];
		//mTrackTitle
	}
}


int Plugin_Key(EventRecord *event) {
	int handled = 0;
	
	if ( gWC -> HandleKey( event->message & charCodeMask ) )
		handled = 1;
	
	return handled;
}


int Plugin_Mouse(EventRecord *inEvent) {
	long  handled = 0;
	Point pt = inEvent -> where;
	::GlobalToLocal( &pt );

	if ( gWC -> PtInTitle( pt ) || ( inEvent -> modifiers & cmdKey ) ) {
		gWC -> SelectConfig();
		handled = 1;
	}

	return handled;
}