/*
	Entheogen API v1.0
	1999, Subband Software
	
	Description:	Entheogen v1.0 API data structures
	Released:		12/06/99

	Version history:
	
	Date		Who		Changes
	--------+---------+------------------------------------------------------
*/

#ifndef H_ENTHEO_API
#define H_ENTHEO_API

#define FFTSIZE  			 512
#define kVIS_API_Version 	 0x100

#define kVIS_PluginFileType 'Plug'
#define kVIS_PluginCreator  'ENTH'

#include <QuickDraw.h>

typedef float keyValue[FFTSIZE];				//	spectral/pcm data for vis

typedef struct VisInfo
{
	/** Control Send **/
	int						mReverse;			//	elapsed/remaining time switch
	/** Control Return **/
	int						mChannels;			//	2=stereo,1=mono
	int						mBitRate;			//	bitrate
	int						mMixRate;			//	mixrate
	int						mStreaming;			//	streaming flag
	char 					mTrackTitle[256];	//	track title or status
	int						mTotalTime;			//	total time in seconds
} VisInfo;

typedef struct {
	void 			 	   *baseAddr; 			// 	address of offscreen buffer
	Rect				   *rBounds;			// 	bounds of bitmap
	SInt32	 			    rowBytes; 			// 	rowBytes (normalized)
	SInt32 				    rDepth;				// 	pixel size (16,32,..)

	keyValue			   *pcm;   				// 	PCM data
	keyValue			   *spectrum;			// 	Spectrum data

	VisInfo 			   *info;				// 	info stuff
	GrafPtr					port;				// 	where to draw, same as GetPort();
} VisInfoBlock;

typedef int		(*vis_control)(int message, int data);

typedef struct
{
	UInt32			signature;					// 	Signature: kVIS_PluginCreator
	UInt32			apiVersion;					//	plugin API version used
	char			*name;						//	human readable plugin name
	char			*version;					//	human readable version
	char			*copyright;					// 	human readable copyright info
	char			*description;				// 	human readable plugin description
	char			*url;						// 	URL
	vis_control		plug_control;				// 	plugin controller routine
	UInt32			reserved;					// 	reserved for misc flags
} vis_plug_header;

enum {
	VIS_CONTROL_INIT 	  = 'INIT',				// 	Called once to initilize plugin
	VIS_CONTROL_TERM	  = 'TERM', 			// 	Called once to shutdown plugin
	VIS_CONTROL_ISENABLED = 'ISEN', 			// 	Is enabled? event
	VIS_CONTORL_ENABLE 	  = 'ENBL', 			// 	Toggle enable event
	VIS_CONTROL_MOUSE	  = 'MOUS', 			// 	Mouse click event
	VIS_CONTROL_KEY		  = 'KEYD', 			// 	Key down event
	VIS_CONTROL_DEPTH	  = 'DPTH', 			// 	Depth change event
	VIS_CONTORL_DRAW	  = 'DRAW', 			// 	Draw event 
	VIS_CONTROL_FULLSCR	  = 'FSCR'  			// 	Full screen switch event
};

#endif