
#include "CEgFileSpec.h"
#include "MacAMP_Visual.h"
#include "WhiteCap.h"
#include "EgOSUtils.h"
#include <Dialogs.h>
#include <Resources.h>
#include <stdlib.h>
#include <time.h>

// This is the gPlugInfo block - a place where MacAmp gets initial information about your plugin.
// Make sure to set the fragment's entry point to gPlugInfo to make it accessible for the MacAmp routines.

VPInfoBlock gPlugInfo =
{
	VP_INFOBLOCK_HEADER( cPluginAuthor, cWhiteCapID),

	"\pWhiteCap",

	VisInitialize,
	VisTerminate,
	nil,				// we don't idle
	VisAbout,
	VisDraw,
	nil,
	
	VisKeyDown,			// we don't want keydowns..
	VisMacEvent,		// ...and events
	
	nil,
	nil,				// we have no settings
	nil,
	nil,
	
	// Nifty macro to set all these 'reserved' values for us
	VP_INFOBLOCK_FOOTER
};




// These are globals.
static WhiteCap*		gWC = NULL;
static short			gRefNum = -1;



/* 
	PlugInitialize

	Called every time use selects your plugin from the Plugins menu. You need to allocate and display a window here,
	init your variables, and do whatever you want, like set the refnum if you need it.
	
	This function uses FSSpec to itself that MacAmp provides to it to gain access to its resource fork.
	
	Other than that, everything is hard-coded and is definitly not an example of good programming =P
*/
OSStatus	VisInitialize(FSSpecPtr inPlugin, WindowPtr* outWindow, UInt32* ) {
	
	gRefNum = ::FSpOpenResFile( inPlugin, fsRdPerm );

	EgOSUtils::Initialize();
		
	CEgFileSpec folder;
	folder.AssignFolder( "WhiteCap Configs" );
	gWC = new WhiteCap( folder );
	
	Rect r = { 0, 0, 0, 0 };
	
	if ( gWC -> BorderlessWindow() )
		*outWindow = ::NewCWindow( NULL, &r, "\pWhiteCap", false, plainDBox,	(WindowPtr)-1, true, NULL );
	else
		*outWindow = ::NewCWindow( NULL, &r, "\pWhiteCap", false, 1985,			(WindowPtr)-1, true, NULL );
		
	// Update whitecap....	
	gWC -> SetWinPort( *outWindow );

	return noErr;
}


/* 
	PlugTerminate

	Called when user deselects your plugin, MacAmp quits or in the case if you have passed visTerminate/visNoMemory error.
	You should dispose your window here & clean up the mess.
*/
OSStatus	VisTerminate(WindowPtr inWin, UInt32*)
{

	delete gWC;
	gWC = NULL;


	::DisposeWindow( inWin );
	
	EgOSUtils::Shutdown();

		
	// Close our resource fork.
	if ( gRefNum != -1 )
		::FSClose( gRefNum );
	
	return noErr;
}


#define SKIP_BINS	10

OSStatus VisDraw(WindowPtr , UInt32*) { 

	// If we detext a click in fullscreen, then exit fullscreen mode no matter what
	if ( gWC -> IsFullscreen() ) {
		if ( ::Button() )
			gWC -> SetFullscreen( false ); 
	}
	
	
	
	int i, j, bin;
	unsigned long time, status = 0;
	unsigned short fftNum;
	double* samp;
	float* sample = gWC -> mSample, binVal;
	long stepsPerBin = gWC -> mStepsPerBin, numBins = gWC -> mNumSampleBins;
	
	
	gPlugInfo.ma->GetFFT( &samp, &fftNum );
	fftNum -= SKIP_BINS;
	if ( fftNum < 0 )
		fftNum = 0; 
	
	if ( gPlugInfo.ma->GetStatus )
		gPlugInfo.ma->GetStatus( &time, &status );

	// Restrict case when we have too many WC bins
	if ( numBins > fftNum )
		numBins = fftNum;
	
	// Limit number of bins if there's not enough mp3 player sample to go around
	else if ( numBins * stepsPerBin > fftNum )
		numBins = fftNum / stepsPerBin;
	
	if ( status & statusStopped )
		gWC -> RecordSample( EgOSUtils::CurTimeMS(), 0 );
	else {
	
		bin = SKIP_BINS;
		for ( i = 0; i < numBins; i++ ) {
			binVal = 0;
			for ( j = 0; j < stepsPerBin; j++, bin++ ) 
				binVal += samp[ bin ];
//			gSample[ i ] = .03 * .0003 * gSample[ i ] * (0.85 + .059 * i);
//			gSample[ i ] = .1 * ( sqrt( 15 + .0003 * gSample[ i ] * (0.85 + .059 * i) ) - sqrt( 15 ) );

			sample[ i ] = sqrt( 40 + .00085 * binVal * (0.85 + .051 * i) ) - sqrt( 40 );
			
			
			//gSample[ i ] = .000013 * gSample[ i ] * (0.85 + .058 * i);
			//gSample[ i ] = 1 * log( 2 * gSample[ i ] + 1 );

			//gSample[ i ] = pow( gSample[ i ], 1.4 );
			//gSample[ i ] = log( 1 + .6 * gSample[ i ] );

		}
		
		sample[ 0 ] *= 0.30;
		sample[ 1 ] *= 0.65;
	}
	
	// If we didn't have enough bins to fill the WC sample, zero out the rest
	for ( i = numBins; i < gWC -> mNumSampleBins; i++ )
		sample[ i ] = 0;

	// Store the newest sample and chuck and old samples
	gWC -> RecordSample( EgOSUtils::CurTimeMS(), .25 );

	// Update the screen, baby.  Let's shag!
	gWC -> Draw();
		
	return noErr;

}


/*
OSStatus VisClick( WindowPtr inWindow, Point inPt, UInt32* ) {

	static long sLastWhen = 0;
	long 	curTime = ::TickCount();
	bool	doSelect = false;
	bool	button = ::Button();

	// see if the cmd key is down
	if ( button ) {
		unsigned char km[16];
		::GetKeys( (unsigned long*) km );
		long code = 55;	 // Cmd key code
		if ( ((km[ code >> 3 ] >> (code & 7)) & 1) != 0 ) {
			doSelect = true;
		}
	}
	
	if ( gWC -> PtInTitle( inPt ) || doSelect )
		gWC -> SelectConfig();
	else if ( curTime - sLastWhen < ::GetDblTime() ) 
		gWC -> SetFullScreen( true );
	else if ( ! gWC -> IsFullScreen() ) {
		if ( button ) {
			Rect r, growBox, lim;
			gWC -> GetWinRect( r );
			growBox = r;
			growBox.left = r.right - 30;
			growBox.top = r.bottom - 30;
			if ( ::PtInRect( inPt, &growBox ) ) {
				lim.top = lim.left = 30;
				lim.bottom = lim.right = 9000;
				unsigned long newSize = ::GrowWindow( inWindow, inPt, &lim );
				::SizeWindow( inWindow, newSize & 0xFFFF, newSize >> 16, false );
				gWC -> GetWinRect( r );
				gWC -> SetWinPort( inWindow, &r );		// SizeWindow changes the size of the win, but gWC doesn't know that
			}
		}
		sLastWhen = curTime;
	}

	return noErr;
}
*/


OSStatus VisKeyDown( WindowPtr, EventRecord* inEvt, UInt32* ) {

	if ( gWC )
		gWC -> HandleKey( inEvt -> message & charCodeMask );	

	return noErr;
}

OSStatus VisMacEvent(WindowPtr inWin, EventRecord* inEvt, UInt32* ) {
	
	gWC -> HandleEvt( inWin, *inEvt );
	
	return noErr;
}





OSStatus	VisAbout(WindowPtr, UInt32*)
{
	::NoteAlert( 556, nil );
	return noErr;
}






/* 
	PlugSettings

	Called when user selects your plugin from the Settings submenu. Only used if you have declared that
	you support settings (flagHasSettings is set).
	
	You should display the settings dialog here, or do whateverr you feel like. =) Just remember that when user
	chooses settings (s)he expects some interactino with the plugin, like the dialog or like that.
*/
OSStatus VisSettings(WindowPtr inWindow, UInt32* ioRefcon) {

#pragma unused (inWindow, ioRefcon)
	return noErr;
}








