
#include "CEgFileSpec.h"
#include "MacAMP_Visual.h"
#include "G-Force.h"
#include "EgOSUtils.h"
#include <Dialogs.h>
#include <Resources.h>
#include <math.h>

// This is the gPlugInfo block - a place where MacAmp gets initial information about your plugin.
// Make sure to set the fragment's entry point to gPlugInfo to make it accessible for the MacAmp routines.


VPInfoBlock gPlugInfo =
{
	VP_INFOBLOCK_HEADER( cPluginAuthor, cGForceID ),

	"\pG-Force",

	VisInitialize,
	VisTerminate,
	nil,				// we don't idle
	VisAbout,
	VisDraw,
	nil,
	
	VisKeyDown,			// we don't want keydowns..
	VisMacEvent,		// ...and events
	
	nil,
	nil,				// we have no settings
	VisTrackBegin,
	nil,
	
	// Nifty macro to set all these 'reserved' values for us
	VP_INFOBLOCK_FOOTER
};

#define SND_BUF_SIZE 512

// These are globals.
static GForce*			gGF = NULL;
static short			gRefNum = -1;
static float			gSampleBuf[ SND_BUF_SIZE ];





/* 
	PlugInitialize

	Called every time use selects your plugin from the Plugins menu. You need to allocate and display a window here,
	init your variables, and do whatever you want, like set the refnum if you need it.
	
	This function uses FSSpec to itself that MacAmp provides to it to gain access to its resource fork.
	
	Other than that, everything is hard-coded and is definitly not an example of good programming =P
*/
OSStatus	VisInitialize(FSSpecPtr inPlugin, WindowPtr* outWindow, UInt32* ) {

	gRefNum = ::FSpOpenResFile( inPlugin, fsRdPerm );


	EgOSUtils::Initialize();

	//PixPort::sOSDepth = 8;
	ScreenDevice::sMinDepth = 8;		

	gGF = new GForce();
	
	Rect r = { 0, 0, 200, 200 };
	if ( gGF -> BorderlessWindow() )
		*outWindow = ::NewCWindow( NULL, &r, "\pG-Force", false, plainDBox,	(WindowPtr)-1, true, NULL );
	else
		*outWindow = ::NewCWindow( NULL, &r, "\pG-Force", false, 1985,		(WindowPtr)-1, true, NULL );
	
	gGF -> SetWinPort( *outWindow );


	return noErr;
}


/* 
	PlugTerminate

	Called when user deselects your plugin, MacAmp quits or in the case if you have passed visTerminate/visNoMemory error.
	You should dispose your window here & clean up the mess.
*/
OSStatus	VisTerminate(WindowPtr inWin, UInt32*)
{


	delete gGF;
	gGF = NULL;


	::DisposeWindow( inWin );
	
	EgOSUtils::Shutdown();

		
	// Close our resource fork.
	if ( gRefNum != -1 )
		::FSClose( gRefNum );
	
	return noErr;
}



OSStatus VisDraw(WindowPtr, UInt32*) { 
	
	unsigned long time, status = 0;
	GrafPtr port;	
	::GetPort( &port );

	// If we detext a click in fullscreen, then exit fullscreen mode no matter what
	if ( gGF -> IsFullscreen() ) {
		if ( ::Button() )
			gGF -> SetFullscreen( false ); 
	}

	short*	samp;
	unsigned short		size;
	gPlugInfo.ma->GetSoundBuffer( (char**) &samp, &size );
	int desired = gGF -> DefaultNum_S_Steps();

	// divide by 4:  2 for num of channels, and 2 for each element is 2 bytes
	int samps = size / 4;


	if ( gPlugInfo.ma->GetStatus )
		gPlugInfo.ma->GetStatus( &time, &status );

	time = EgOSUtils::CurTimeMS();

	if ( status & statusStopped || samps < 10 )
		gGF -> RecordZeroSample( time );
	else {

		if ( samps > desired )
			samps = desired;
		if ( samps > SND_BUF_SIZE )
			samps = SND_BUF_SIZE;
			
		// divide by 4:  2 for num of channels, and 2 for each element is 2 bytes
		for ( int i = 0; i < samps; i++ ) {
			gSampleBuf[ i ] = samp[ 2*i ];
		}
			
		gGF -> RecordSample( time, gSampleBuf, .000029, samps );
	}
	
	::SetPort( port );
	
	return noErr;

}




OSStatus VisKeyDown( WindowPtr, EventRecord* inEvt, UInt32* ) {
	
	if ( gGF )
		if ( gGF -> HandleKey( inEvt -> message & charCodeMask ) )
			return 1;

	return noErr;
}


OSStatus VisMacEvent(WindowPtr inWind , EventRecord* inEvt, UInt32* ) {

		
	switch ( inEvt -> what ) {
			
		case updateEvt:	
		case mouseDown:	
			gGF -> HandleEvt( inWind, *inEvt );
			break;
	}


	return noErr;
}





OSStatus	VisAbout(WindowPtr, UInt32*)
{
	::NoteAlert( 556, nil );
	return noErr;
}



OSStatus	VisTrackBegin(WindowPtr, UInt32*) {
	Str255 artist, title, album, year;
	OSStatus err;

	if ( gGF ) {
		err = gPlugInfo.ma -> GetID3Information( artist, title, album, year, NULL );

		if ( err != resNotFound ) {
			gGF -> mArtist			.Assign( artist );
			gGF -> mAlbum			.Assign( album );
			gGF -> mSongTitle		.Assign( title );
			gGF -> NewSong();
		}
	}
	
	return noErr;
}




/* 
	PlugSettings

	Called when user selects your plugin from the Settings submenu. Only used if you have declared that
	you support settings (flagHasSettings is set).
	
	You should display the settings dialog here, or do whateverr you feel like. =) Just remember that when user
	chooses settings (s)he expects some interactino with the plugin, like the dialog or like that.
*/
OSStatus VisSettings(WindowPtr inWindow, UInt32* ioRefcon) {

#pragma unused (inWindow, ioRefcon)
	return noErr;
}






