
//============================================================================
//----------------------------------------------------------------------------
//								VisFramework.h
//								version 1.0.6
//								Mike Wright
//								e-mail questions to darwin@mbay.net
//
//	Header file for the VisFrameworkHandlers.c and VisFrameworkMain.c.
//
//----------------------------------------------------------------------------
//============================================================================

#pragma once

#include "SoundJamAPI.h"
#include "VisualPlugin.h"

/*==================================================================================================
									Typedefs
==================================================================================================*/

/*======================================================================*/
/* This is the struct that contains the global data that is stored in	*/
/* the refcon, restored in VisHandler(), and passed by VisHandler() to  */
/* the handler  routines as *plugInData.								*/
/*																		*/
/* Fields have been included that match	all the fields of the various 	*/
/* structs associated with the various messages. They are filled in by 	*/
/* the handler routines for those messages.								*/
/*======================================================================*/

class WhiteCap;

struct VisHandlerData
{
	/* VisualPluginInitMessage data*/
	void				*appCookie;
	SoundJamProcPtr		playerProc;
	UInt32				version;
	
	/* VisualPluginShowWindowMessage  and VisualPluginSetWindowMessage data */
	CGrafPtr			port;
	Rect				drawRect;
	
	/* VisualPluginPlayMessage data */
	TrackSpec			*trackSpec;
	SInt32				volume;
	UInt32				bitRate;
	SoundComponentData	soundFormat;
	
	/* VisualPluginRenderMessage data */
	RenderVisualData	renderData;

	/* VisualPluginProcessSamplesMessage */
	UInt32				timeStampID;
	SInt16				*sampleBuffer;		/* the data in this buffer may be modified by the plugin	*/
	UInt32				numSamples;
	UInt32				maxSamples;
	UInt32				numOutputSamples;	/* must be less than or equal to maxSamples 				*/
	
	/* framework-specific data */
	OSType				magic;				/* used to identify our plugin for debugging purposes		*/
	bool				mPlayingMusic;
	bool				mIsVisible;
	WhiteCap*			mWC;
};
typedef struct VisHandlerData VisHandlerData;

void HandlePlugInMouseDown (VisHandlerData *plugInData,EventRecord *theEvent);


/*==================================================================================================
									Function Prototypes
==================================================================================================*/

/* functions in VisFrameworkMain.c */
OSStatus main (OSType message, PluginMessageInfo *messageInfo, void *refcon);
OSStatus VisHandler (OSType message, VisualPluginMessageInfo *messageInfo, void *refcon);

/* functions in VisFrameworkHandlers.c */
OSStatus HandleInitializeMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
OSStatus HandleCleanupMessage (VisHandlerData *plugInData);
OSStatus HandleEnableMessage (VisHandlerData *plugInData);
OSStatus HandleDisableMessage (VisHandlerData *plugInData);
OSStatus HandleShowWindowMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
OSStatus HandleHideWindowMessage (VisHandlerData *plugInData);
OSStatus HandleResizeMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
OSStatus HandleSetWindowMessage (VisualPluginMessageInfo *messageInfo,VisHandlerData *plugInData);
OSStatus HandlePlayMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
OSStatus HandleStopMessage (VisHandlerData *plugInData);
OSStatus HandlePauseMessage (VisHandlerData *plugInData);
OSStatus HandleUnpauseMessage (VisHandlerData *plugInData);
OSStatus HandleRenderMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
OSStatus HandleProcessSamplesMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
OSStatus HandleFlushSamplesMessage (VisHandlerData *plugInData);
OSStatus HandleEventMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
OSStatus HandleUpdateMessage (VisHandlerData *plugInData);
OSStatus HandleIdleMessage (VisHandlerData *plugInData);
OSStatus HandleAboutMessage (VisHandlerData *plugInData);
OSStatus HandleConfigureMessage (VisualPluginMessageInfo *messageInfo, VisHandlerData *plugInData);
