#include <Types.h>
#include <Memory.h>
#include <Quickdraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Menus.h>
#include <Windows.h>
#include <Devices.h>



#include "CEgFileSpec.h"
#include "WhiteCap.h"
#include "EgOSUtils.h"
#include "SoundInputPort.h"








// Prototypes
OSErr InstallRequiredAppleEvents (void);
pascal OSErr HandleOApp (AppleEvent *theAppleEvent, AppleEvent *reply, long handlerRefcon);
pascal OSErr HandleQuit (AppleEvent *theAppleEvent, AppleEvent *reply, long handlerRefcon);
void Initialize(void);
void			 MenuBarInit();

// Globals
int				gQuitting;
WhiteCap*		gWC;
WindowPtr		gWC_Win;







static void HandleQTConvert() {	
	bool ok;
	CEgFileSpec spec;
	CEgIFile iFile;

	ScreenDevice::sMinDepth = 32;

	ok = EgOSUtils::AskOpen( NULL, spec, cWC_CaptureType );
	if ( ok ) {
		iFile.open( &spec );
		if ( iFile.noErr() ) {
			ok = false;
			
			if ( iFile.GetLong() == (long) cWhiteCapID ) {
				if ( iFile.GetLong() == WHITECAP_COMPAT_VERSION )
					gWC -> ExportQT( &iFile );
			}
		}
	}
		
	ScreenDevice::sMinDepth = 16;
}




static OSErr DispatchMenuChoice (long menuChoice) {
	OSErr						err					= noErr;
	long						menu;
	long						item, accNumber;
	MenuHandle					appleMenu;
	Str255						accName;
	
	if (menuChoice != 0) {
		menu = ((unsigned long) menuChoice) >> 16;
		item = menuChoice & 0xFFFF;
		switch (menu) {
			case 128:		// Apple Menu
				if ( item == 1 ) 
					::NoteAlert( 556, nil );
				else {
					appleMenu = GetMenuHandle (128);
					GetMenuItemText (appleMenu, item, accName);
					accNumber = OpenDeskAcc (accName);
				}
				break;
			case 129:		// File Menu
				switch (item) {
					case 1: // Convert to QT
						HandleQTConvert();
						break;
						
					case 3:	// Quit
						gQuitting = true;
						break;
				}
				break;
				
			case 131:
				switch ( item ) {
					case 1: // WhiteCap Win
						if ( ! gWC -> IsFullscreen() ) {
							if ( ( (WindowPeek) gWC_Win ) -> visible )
								::HideWindow( gWC_Win );
							else
								::ShowWindow( gWC_Win );
						}
						break;
						
					case 3: // Fullscreen
						gWC -> SetFullscreen( ! gWC -> IsFullscreen() );
						break;
				}
				break;
		}
	}

	HiliteMenu (0);

	return err;
}






//MW specified argument and return type.
int main(void)
{

	OSErr						err;
	EventRecord					event;
	WindowPtr					wind;
	SoundInput  				soundPort;
	int i;
	float						binRange;
	bool 						inForeground = true, isVisible;

	Initialize();
	

		
	CEgFileSpec folder;
	folder.AssignFolder( "WhiteCap Configs" );
	gWC = new WhiteCap( folder );
	
	Rect r = { 0, 0, 0, 0 };
	if ( gWC -> BorderlessWindow() )
		gWC_Win = ::NewCWindow( NULL, &r, "\pWhiteCap", false, plainDBox,	(WindowPtr)-1, true, NULL );
	else
		gWC_Win = ::NewCWindow( NULL, &r, "\pWhiteCap", false, 1985,		(WindowPtr)-1, true, NULL );

	gWC -> SetWinPort( gWC_Win );

	soundPort.SetSampleSkip( 3 );
	err = soundPort.StartRecording();
	gQuitting = err != noErr;

	Rect lim;
	
	while ( ! gQuitting ) {
		isVisible = ( (WindowPeek) gWC_Win ) -> visible;
		
		::WaitNextEvent (everyEvent, &event, ( inForeground && isVisible ) ? 0 : 20, NULL );

		switch ( event.what ) {
			case kHighLevelEvent:
				err = AEProcessAppleEvent (&event);
				break;
				
			case keyDown:
				if (event.modifiers & cmdKey)
					err = DispatchMenuChoice (MenuKey (event.message & charCodeMask));
				else
					gWC -> HandleKey( event.message  & charCodeMask );	
				break;
				
			case osEvt:
				switch ((event.message >> 24) & 0x00FF) {		// get high byte of message
					case suspendResumeMessage:
					
						// set the foreground/background state
						inForeground = ((event.message & resumeFlag) != 0);
						break;
					
					case mouseMovedMessage:
						break;
				}
				break;
								
			case mouseDown:
				i = ::FindWindow (event.where, &wind );
				switch ( i ) {
					case inMenuBar:
						::CheckItem( GetMenuHandle( 131 ), 1, isVisible );
						DispatchMenuChoice(MenuSelect (event.where));
						break;
						
					case inDrag:
						lim.top = lim.left = 0;
						lim.bottom = lim.right = 29000;
						::DragWindow( gWC_Win, event.where, &lim );
						break;
						
					case inGoAway:
						if ( ::TrackGoAway( gWC_Win, event.where ) )
							::HideWindow( gWC_Win );
						break;
						
					default:
						gWC -> HandleEvt( gWC_Win, event );
						break;
				}
				break;
				
			case updateEvt:
				::BeginUpdate( gWC_Win );
				gWC -> Refresh();
				::EndUpdate( gWC_Win );
				break;
		}
	
		// If the WC window is visible then process a sound sample
		if ( isVisible || gWC -> IsFullscreen() ) {
		
			// Get a freq specrum sample
			binRange = ( (float) gWC -> mStepsPerBin ) / 7.0;
			soundPort.GetSpectrum( gWC -> mNumSampleBins, binRange, gWC -> mSample );
			
			// Record the freq sample in whitecap
			gWC -> RecordSample( EgOSUtils::CurTimeMS(), .00039 );

			// Update the screen, baby.  Let's shag!
			gWC -> Draw();		
		}
	}


	

	delete gWC;


	::DisposeWindow( gWC_Win );
	
	EgOSUtils::Shutdown();

	return 0;	
}






// 
//	Initialize everything for the program, make sure we can run
//

//MW specified argument and return type.
void Initialize(void)
{
	
	/* Initialize all the needed managers. */
	MaxApplZone ();
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();

	::EnterMovies();
	
	Handle					menuBar;
	MenuHandle				menu;
	OSErr					err				= noErr;

	err = InstallRequiredAppleEvents ();

	menuBar = GetNewMBar (128);
	if (menuBar != nil) {
		SetMenuBar (menuBar);
		menu = GetMenuHandle (128);
		if (menu != nil) {
			AppendResMenu (menu, 'DRVR');
			DrawMenuBar ();
		} else {
			err = memFullErr;
		}
	}

	EgOSUtils::Initialize();
}




OSErr InstallRequiredAppleEvents (void) {
	OSErr		err;

	err = AEInstallEventHandler (kCoreEventClass, kAEOpenApplication, NewAEEventHandlerProc (HandleOApp), 0, false);


	err = AEInstallEventHandler (kCoreEventClass, kAEQuitApplication, NewAEEventHandlerProc (HandleQuit), 0, false);

	return err;
}

pascal OSErr HandleOApp (AppleEvent *theAppleEvent, AppleEvent *reply, long handlerRefcon) {
#pragma unused (theAppleEvent, reply, handlerRefcon)

	return noErr;				/* We're up and running */
}





pascal OSErr HandleQuit (AppleEvent *theAppleEvent, AppleEvent *reply, long handlerRefcon) {
#pragma unused (theAppleEvent, reply, handlerRefcon)

	gQuitting = true;
	return noErr;
}



