#pragma once


#include "WhiteCapWorld.h"
#include "PixPort.h"
#include "FileSpecList.h"
#include "Prefs.h"
#include "XLongList.h"
#include "CEgIOFile.h"
#include "ScreenDevice.h"


#if EG_MAC
#include <Files.h>
#include <Events.h>
#endif

#define cWhiteCapID		'WCap'


// Capture codes
#define CAPTURE_SAMPLE	55
#define CAPTURE_CONFIG_CHANGE	56
#define CAPTURE_CONFIG_MORPH	57


class WhiteCap {

	public:	
		
		// Once you make a new WhiteCap, don't do anything else until you call SetPort() or SetWinRect()
								WhiteCap( CEgFileSpec& inPluginsFolder, void* inRefCon = NULL );
								~WhiteCap();

		// Calling this assumes that the GrafPort WhiteCap has is a window.  This call simply is a 
		// MacOS/Win32 util to get a rect for the OS window size
		void					GetWinRect( Rect& outRect );
		
		// Tell's WhiteCap, "work inside this grafPort and keep your slimy butt inside inRect"
		// inAtFullsceen is needed because WC responds to input differently if it's fullscreen mode vs. no.
		void					SetPort( GrafPtr inPort, const Rect& inRect, bool inAtFullsceen );
		
		// Use this call (instead of SetPort()) when WhiteCap's
		// is in control of its own window (ie, in MacAMP and Winamp)--this call will make a call to
		// SetPort().  In other words, this call is like SetPort() but it also
		// correctly positions WhiteCap's window for you, changing inRect from global cords to local cords
		// If inRect is invalid or NULL, the prefs are used 
		void					SetWinPort( WindowPtr inPort, const Rect* inRect = NULL );
		

		// Tells whitecap: "here's a sound sample and the time is X.  the sample data is in sSample[]"
		void					RecordSample( long inTime, float inScale );

		// Draws the plugin to the screen, at the time index used in RecordSample().  The part of the
		// screen refreshed is the union of the changed rect since the last call to Draw() and
		// any rects passed to RefreshRect() since the last call to Draw().
		void					Draw();
		
		// This function redirects drawing that WhiteCap will do to the given frame buffer.
		void					RedirectFrame( void* inBits, long inBytesPerRow, long inBitDepth, long inX, long inY )  { mPort.RedirectDrawing( inBits, inBytesPerRow, inBitDepth, inX, inY ); }
		
		// Returns true if key was handled/caught
		bool					HandleKey( long inChar );
	
		void					SetFullscreen( bool inFullScreen );
		inline bool				IsFullscreen()		{ return mAtFullScreen;	}

		void					StoreWinRect()							{ if ( ! IsFullscreen() ) GetWinRect( mWinRectHolder ); }
		
		void					Refresh();
		
		void					SelectConfig();

		void					ToggleFrameCounter()					{ mFrameCount = (mFrameCount >= 0) ? -1 : 0;	}


		bool					PtInTitle( Point inPt );
		
		bool					BorderlessWindow()						{ return mBorderlessWind;						}


		Point					GetFullscreenSize()						{ return mFullscreenSize;						}
		
		void					ExportQT( CEgIFile* inStream );
		
		// Deposit  new sound sample in this array when calling RecordSample()
		float*					mSample;


		// Public, global Plugin prefs
		long					mNumSampleBins;
		long					mStepsPerBin;	
	


		#if EG_MAC
		long					HandleEvt( WindowPtr inWin, EventRecord& evt );
		#endif
		
	protected:
	
		
		void					EnableSlideShow();
	
		long					mLastRecordTime;	// Holds the 'current' time
		long					mBaseTimeIndex;		// The time when the plugin was started -- the time index that is 0
		
		bool					mDoingSetPortWin;	// true when a thread is currently inside SetPortWin()
		Rect					mWinRectHolder;		// Win rect holder while we're n FS mode
		Rect					mPaneRect;			// Local cords rect within mOSPort we can draw in
		FileSpecList			mConfigs;
		int						mCurConfigNum;
		XLongList				mConfigPlayList;
		PixPort					mPort;
		long 					mNextShapeChange;
		bool					mInSlideShowMode;
		GrafPtr					mOutPort;
		WindowPtr				mWind;
		Prefs					mPrefs;
	
		// Keyboard info
		UtilStr					mKeyBuf;
		long					mLastKeyTime;
		
		// Plugin prefs
		long					mSlideShowInterval;
		long					mScrnSaverDelay;
		long					mTransitionLo;
		long					mTransitionHi;
		long					mShwT_Pref;
		long					mBorderlessWind;
		long					mHandleKeys;
		long					mAddDefaultConfig;
		float					mMagScale;
		Point					mFullscreenSize;
		long					mFullscreenDepth;
		long					mFullscreenDevice;
		long					mFullscreenFreq;
		UtilStr					mKeysIgnored;
		float					mSmoothFactor;

	
		ScreenDevice			mScreen;

		// For dumping raw spectrum data
		bool					mCaptureMode;		
		CEgIOFile				mDumpFile;
				
		#define MAX_WORLDS		10
		long					mNumWorlds;	
		WhiteCapWorld*			mWorld[ MAX_WORLDS ];

		// Stuff dealing with full screen mode
		bool					mAtFullScreen;
		bool					mMouseWillAwaken;	// true if moving the mouse will exit fullscreen mode
		Point					mLastMousePt;
		long					mLastActiveTime, mLastKeyPollTime;
		KeyMap					mCurKeys, mPastKeys;
		
		// Frame rate/calc members.  mFrameCount >= 0 if frame counting/displaying is on
		UtilStr					mFrameRate;
		long					mFramesPer10Secs;
		long					mFrameCountStart;
		long					mFrameCount;
		
		void					BuildConfigList( CEgFileSpec& inPluginsFolder );
		void					LoadConfig( int inConfigNum, long inCurTime, bool inAllowMorph );
		void					LoadNextConfig();
		
		//  Chops up mDispRect into smaller rects, making a rect for each config to be shown.
		void					ResizeWorlds();
		
		void					HandleKybdBuf();

		bool					PlaybackSample( CEgIStream* inStream, long* outTimeIndex = NULL );
		
		UtilStr					mSampleBuf;
		void					SetNumSampleBins( long inNumBins );
		void *					mRefCon;
};