#include "Sample.h"

#include "UtilStr.h"
#include "V3.h"
#include "R3Matrix.h"
#include "WhiteCap.h"

UtilStr	Sample::sTemp;


Sample::Sample() {
	mNext			= NULL;
	mSampleTime		= -1;
	mSample			= NULL;
}




#define BINS_TO_COMPARE	15

void Sample::Assign( long inCurTime, const float inSample[], long inNumBins, const Sample* inPrev, float inFalloff ) {
	const float*	prev = NULL;
	float*			samp;
	float			w, bin, sum;
	long			j, i, idx, realloc;
	bool			done = false;
	
	mSampleTime = inCurTime;
	
	if ( mSample )
		realloc = mSample -> mNumFcnBins != inNumBins;
	else
		realloc = true;
		
	if ( realloc ) {

		mSample = (ExprUserFcn*) mBuf.Dim( sizeof( ExprUserFcn ) + sizeof( float ) * inNumBins );
		mSample -> mNumFcnBins = inNumBins;
	}
	
	samp = mSample -> mFcn;
	
	if ( inPrev ) {
		w = inFalloff * ((float) ( mSampleTime - inPrev -> mSampleTime ));
		if ( w < 1.0 ) {
			prev = inPrev -> mSample -> mFcn;
			for ( i = 0; i < inNumBins; i++ ) {
				bin = inSample[ i ];
				samp[ i ] = w * bin + (1.0 - w) * prev[ i ];
			}
			done = true;
		} 
	}
	
	if ( ! done )
		UtilStr::Move( samp, inSample, sizeof( float ) * inNumBins );
	
	// Calculate bass1
	// Assign mBassLvl here -- choose the larger bins
	char* chosen = sTemp.Dim( BINS_TO_COMPARE );
	for ( i = 0; i < BINS_TO_COMPARE; i++ )
		chosen[ i ] = 0;

	// sum the 4 largest bins...
	sum = 0;
	for ( j = 0; j < 4; j++ ) {
	
		// Default to the bin not taken yet
		for ( idx = 0; chosen[ idx ]; idx++ ) { 
		}
		bin = inSample[ idx ];
		
		// Find next largest bin, then 'choose' it
		for ( i = idx + 1; i < BINS_TO_COMPARE; i++ ) {
			if ( samp[ i ] > bin && chosen[ i ] == 0 ) {
				bin = samp[ i ];
				idx = i;
			}
		}
		
		sum += bin;
		chosen[ idx ] = 1;
	}
	mBass[ 0 ] = sum;
	
	
	// Calculate bass2 -- sum some low freq bins
	sum = 0;
	for ( i = 0; i < inNumBins >> 2; i++ )
		sum += samp[ i ];
	mBass[ 1 ] = sum;
	
	
	// Calculate bass3 -- the differential average value
	sum = 0;
	if ( inPrev ) {
		if ( mSampleTime - inPrev -> mSampleTime < 500 ) {
			w = 1.0 / ( (float) ( inCurTime - inPrev -> mSampleTime ) );
			prev = inPrev -> mSample -> mFcn;
			for ( i = 0; i < inNumBins / 3; i++ ) {
				bin = samp[ i ] - prev[ i ];
				if ( bin > 0 )
					sum += w * bin;
			}
		}
	}
	mBass[ 2 ] = sum;
}
		
		
	/*
void Sample::GenerateCords( V3* ioPts, float* inTail, V3& inOffset, R3Matrix& inT, float inScale, float inPersIntcpt ) {
	float mag, x, y, z;

	for ( int i = 0; i < NUM_SAMPLE_BINS; i++ ) {
		mag = mSample[ i ];
		x =		*inTail;		inTail++;
		y =		*inTail;		inTail++;
		z =		*inTail;		inTail++;
		x +=	mag * *inTail;	inTail++;
		y +=	mag * *inTail;	inTail++;
		z +=	mag * *inTail;	inTail++;
		ioPts -> mX = x - inOffset.mX;
		ioPts -> mY = y - inOffset.mY;
		ioPts -> mZ = z - inOffset.mZ;
		ioPts -> transform( inT );
		ioPts -> mX = inScale * ioPts -> mX / ( ioPts -> mZ + inPersIntcpt );
		ioPts -> mY = inScale * ioPts -> mY / ( ioPts -> mZ + inPersIntcpt );
		ioPts++;
	}
}
*/

