#include "WhiteCap.h"

#include "EgOSUtils.h"
#include "XFloatList.h"

#if EG_MAC
#include <Menus.h>
#include <Windows.h>
#include <Palettes.h>

#if MACAST
#include "MacAMP_Visual.h"
extern VPInfoBlock gPlugInfo;
#endif


#if SOUNDJAM
#include "VisFramework.h"
#endif


#include "CEgFileSpec.h"
#include "Sample.h"
#include "CEgIOFile.h"



#define __setupPort		GDHandle		saveDev;								\
 						GrafPtr			savePort;								\
						::GetGWorld( (GWorldPtr*)(&savePort), &saveDev );		\
						::SetPort( mOutPort );


#define __restorePort 	::SetGWorld( (GWorldPtr)(savePort), saveDev );			

#define __drawText( x, y, str )		::MoveTo( x, y );							\
									::DrawString( str -> getPasStr() );

#endif



#if WINAMP || ( EG_WIN && STANDALONE )
#define __setupPort					if ( ! IsFullscreen() )						\
										mOutPort = ::GetDC( mWind );


#define __restorePort 				if ( ! IsFullscreen() )						\
										::ReleaseDC( mWind, mOutPort );

#include "Winamp_VIS.H"
extern winampVisModule gWCModule;
#endif



#if SONIQUE
#define __setupPort					mOutPort = (HDC) mWind;	
#define __restorePort 				mOutPort = NULL;
#endif


#if EG_WIN
#include "RectUtils.h"

#endif



WhiteCap::WhiteCap( CEgFileSpec& inPluginsFolder, void* inRefCon ) :
#if SONIQUE
	mPrefs( "\\Vis\\WhiteCap\\WhiteCap Prefs.txt", false ),
#elif WINAMP
	mPrefs( "WhiteCap Prefs.txt", false ),
#elif STANDALONE && EG_WIN
	mPrefs( "WhiteCap Prefs (Standalone).txt", false ),
#elif STANDALONE
	mPrefs( "WhiteCap Prefs (Standalone)", true ),
#elif SOUNDJAM
	mPrefs( "WhiteCap Prefs (SoundJam)", true ),
#elif MACAST
	mPrefs( "WhiteCap Prefs (Macast)", true ),
#elif ENTHOGEN
	mPrefs( "WhiteCap Prefs (Enthogen)", true ),
#endif
	mConfigs( cNoDuplicates_CaseInsensitive, cSortLowToHigh) {
	UtilStr name, str;
	
	
	mPrefs.Load();
	if ( mPrefs.GetPref( 'Vers' ) != WHITECAP_COMPAT_VERSION ) {
		mTransitionLo		= 6;
		mTransitionHi		= 23;
		mSlideShowInterval	= 11000;		// Factory: 10 secs per config
		mScrnSaverDelay		= -1 * 60000.0;	// Factory: screen saver mode disabled
		mPrefs.SetPref( 'Vers', WHITECAP_COMPAT_VERSION );
		mShwT_Pref			= 0x3;
		mBorderlessWind		= 0;
		mHandleKeys			= 1;
		mAddDefaultConfig	= 1;
		mMagScale 			= 1;
		mNumSampleBins		= 0;
		mStepsPerBin		= 0;
		mFullscreenSize.h	= 640;
		mFullscreenSize.v	= 480;
		mFullscreenDepth	= ScreenDevice::sOSDepth;
		mFullscreenDevice	= 0;
		mFullscreenFreq		= 0;
		
		if ( mSmoothFactor < .1 ) {
	
			#if STANDALONE	
			mSmoothFactor		= .55;  // done (windows)
			#elif SOUNDJAM || MACAST
			mSmoothFactor		= .4; 
			#elif SONIQUE
			mSmoothFactor		= .7;  // done
			#elif WINAMP
			mSmoothFactor		= .48; // done
			#endif
		} }
		
	else {
		mMagScale			= mPrefs.GetPref( 'MScl' ) / 1000.0;
		mSmoothFactor		= mPrefs.GetPref( 'GSmo' ) / 1000.0;
		mTransitionLo		= mPrefs.GetPref( 'TrLo' );
		mTransitionHi		= mPrefs.GetPref( 'TrHi' );
		mSlideShowInterval	= mPrefs.GetPref( 'Slde' ) * 1000.0;
		mScrnSaverDelay		= mPrefs.GetPref( 'SSvr' ) * 1000.0 * 60.0;
		mShwT_Pref			= mPrefs.GetPref( 'ShwT' );
		mBorderlessWind		= mPrefs.GetPref( 'NoBo' );
		mHandleKeys			= mPrefs.GetPref( 'Kybd' );
		mAddDefaultConfig	= mPrefs.GetPref( 'Dflt' );
		mNumSampleBins		= mPrefs.GetPref( 'Stps' );
		mStepsPerBin		= mPrefs.GetPref( 'SBin' );
		mFullscreenSize.h	= mPrefs.GetPref( 'FS_X' );
		mFullscreenSize.v	= mPrefs.GetPref( 'FS_Y' );
		mFullscreenDepth	= mPrefs.GetPref( 'FS_D' );
		mFullscreenDevice	= mPrefs.GetPref( 'FS_#' );
		mFullscreenFreq		= mPrefs.GetPref( 'FS_F' );
		mPrefs.GetPref( 'KIgn', mKeysIgnored );
	}
	
	mRefCon				= inRefCon;
	mInSlideShowMode	= true;
	mAtFullScreen		= false;
	mWind				= NULL;
	mOutPort			= NULL;
	mNumWorlds			= 0;
	mFrameCount			= 0;
	mLastKeyPollTime	=
	mBaseTimeIndex		=
	mLastActiveTime 	= EgOSUtils::CurTimeMS();
	mFrameCountStart	=
	mLastRecordTime		= 0;
	mCurConfigNum		= -1;
	mNextShapeChange	= 0x7FFFFFFF;
	mFramesPer10Secs	= 400;
	mDoingSetPortWin	= false;
	mMouseWillAwaken	= false;
	mSample				= NULL;
	mCaptureMode		= false;


	// Prevent any bad prefs...
	if ( mStepsPerBin <= 0 || mNumSampleBins <= 0 ) {
		mNumSampleBins		= 47;
		
		#if MACAST
		mStepsPerBin		= 6;  // done
		#elif EG_MAC && STANDALONE
		mStepsPerBin		= 5;  // done
		#elif SOUNDJAM
		mStepsPerBin		= 4;
		#elif SONIQUE || ENTHOGEN
		mStepsPerBin		= 3;  // done
		#else
		mStepsPerBin		= 7;  // done  (winamp, windows standalone)
		#endif
	}
	

	
	SetNumSampleBins( mNumSampleBins );

	BuildConfigList( inPluginsFolder );
}




WhiteCap::~WhiteCap() {


	SetFullscreen( false );
			
	// Rewrite the prefs to disk...
	mPrefs.SetPref( 'Slde', mSlideShowInterval / 1000.0 );
	mPrefs.SetPref( 'SSvr', mScrnSaverDelay / 60000.0 );
	mPrefs.SetPref( 'TrHi', mTransitionHi );
	mPrefs.SetPref( 'TrLo', mTransitionLo );
	mPrefs.SetPref( 'MScl', mMagScale * 1000 );
	mPrefs.SetPref( 'GSmo', mSmoothFactor * 1000 );
	mPrefs.SetPref( 'ShwT', mShwT_Pref );
	mPrefs.SetPref( 'NoBo', mBorderlessWind );
	mPrefs.SetPref( 'Dflt', mAddDefaultConfig );
	mPrefs.SetPref( 'Kybd', mHandleKeys	);
	mPrefs.SetPref( 'Stps', mNumSampleBins );
	mPrefs.SetPref( 'SBin', mStepsPerBin );	
	mPrefs.SetPref( 'FS_X', mFullscreenSize.h );
	mPrefs.SetPref( 'FS_Y', mFullscreenSize.v );	
	mPrefs.SetPref( 'FS_D', mFullscreenDepth );	
	mPrefs.SetPref( 'FS_#', mFullscreenDevice );
	mPrefs.SetPref( 'FS_F', mFullscreenFreq );
	mPrefs.SetPref( 'KIgn', mKeysIgnored );


	Rect r;
	#if ! ( WINAMP || ( EG_WIN && STANDALONE ) ) 
	GetWinRect( r );
	#else
	// Very annoying:  in Win32, the call to GetWindowRect() is returning garbage in ~WhiteCap(), so just 
	// forget about saving the window position for now in windows :_(
	
	//GetWinRect( r );
	r = mWinRectHolder;  // Just use a recent rect for now
	#endif
	mPrefs.SetPref( 'wTop', r.top );
	mPrefs.SetPref( 'wLft', r.left );
	mPrefs.SetPref( 'wBot', r.bottom );
	mPrefs.SetPref( 'wRgt', r.right );

	
	mPrefs.Store();	
	
	// Delete all the existing worlds
	for ( int i = 0; i < mNumWorlds; i++ )
		delete mWorld[ i ];
}





bool WhiteCap::PtInTitle( Point inPt ) {
	int i;
	
	Rect r;
	GetWinRect( r );
	
	inPt.v -= r.top;
	inPt.h -= r.left;
		
	for ( i = 0; i < mNumWorlds; i++ ) {
		if ( ::PtInRect( inPt, &mWorld[ i ] -> mTitleRect ) )
			return true;
	}
		
	return false;
}

#define SUBMENU_SIZE 17




void WhiteCap::SelectConfig() {
	long i, sel;
	Point pt;
	UtilStr name;

	EgOSUtils::GetMouse( pt );
			
	// If we're in full screen mode, show the mouse ptr
	if ( mAtFullScreen )
		EgOSUtils::ShowCursor();
		
	#if EG_MAC
	MenuHandle	curSubMenu = ::NewMenu( 55, "\pBlah" ), menu;
	long groupNum, num = mConfigs.Count();

	__setupPort
	
	// If we found a world, show a popup menu to select from the configs
	menu = ::NewMenu( 55, "\p " );
	
	// Add the stard slideshow item
	::AppendMenu( menu, "\p " );
	::SetMenuItemText( menu, 1, "\p(Start Slide Show)" );
	if ( mInSlideShowMode )
		::DisableItem( menu, 1 );

	for ( i = 1; i <= num; i++ ) {
	
		groupNum = 1 + i / SUBMENU_SIZE;
		
		// Split up the config list into groups of submenus
		if ( i % SUBMENU_SIZE == 1 ) {
			
			// Make the submenu
			curSubMenu = ::NewMenu( 55 + groupNum, "\pBlah" );
			::InsertMenu( curSubMenu, hierMenu );

			// Contruct and rename the title string
			name.Assign( "Group " );
			name.Append( groupNum );
			::AppendMenu( menu, "\p " );
			::SetMenuItemText( menu, 1 + groupNum, name.getPasStr() );
			
			// Tell MacOS to make it a heir menu
			::SetItemCmd( menu, 1 + groupNum, hMenuCmd );
			::SetItemMark( menu, 1 + groupNum, 55 + groupNum );
		}
					
		// Get the name for the config number and set the menu item to that name
		mConfigs.FetchSpecName( i, name );
		::AppendMenu( curSubMenu, "\p " );
		sel = 1 + ( i - 1 ) % SUBMENU_SIZE;
		::SetMenuItemText( curSubMenu, sel, name.getPasStr() );
		
		if ( mCurConfigNum == i )
			::CheckMenuItem( curSubMenu, sel, true ); 
	}
		
	::InsertMenu( menu, -1 );
	sel = ::PopUpMenuSelect( menu, pt.v - 6, pt.h, 1 );
	
	// Figure out what config number was selected
	i = sel >> 16;
	if ( i )
		sel = ( i - 56 ) * SUBMENU_SIZE + (sel & 0xFFFF) + 1;
	else
		sel = 0;
	
	//  Cleanup
	i = 0;
	do {
		menu = ::GetMenuHandle( 55 + i );
		if ( menu ) {
			::DeleteMenu( 55 + i );
			::DisposeMenu( menu );
		}
		i++;
	} while ( menu );
	#endif
	
	
	#if EG_WIN
	long flags, numSubMenus = 0;
	long num = mConfigs.Count();
	
	HMENU curSubMenu = NULL, myMenu = ::CreatePopupMenu();
	
	// Add item to start slideshow
	flags = MF_STRING;
	if ( mInSlideShowMode )
		flags |= MF_GRAYED;
	else
		flags |= MF_ENABLED;
	::AppendMenu( myMenu, flags, 1, "(Start Slide Show)" );
	
	for ( i = 1; i <= num; i++ ) {
	
		// Split up the config list into groups of submenus
		if ( i % SUBMENU_SIZE == 1 ) {
			curSubMenu = ::CreatePopupMenu();
			
			// Contruct the submenu title string
			name.Assign( "Group " );
			name.Append( 1 + i / SUBMENU_SIZE );
			::InsertMenu( myMenu, 0xFFFFFFFF, MF_POPUP | MF_STRING | MF_BYPOSITION, (long) curSubMenu, name.getCStr() );
		}
		
		// Set flags for the item
		flags = MF_ENABLED | MF_STRING;
		if ( mCurConfigNum == i )
			flags |= MF_CHECKED;
			
		// Get the name for the config number and set the menu item to that name
		mConfigs.FetchSpecName( i, name );
		::AppendMenu( curSubMenu, flags, i + 1, name.getCStr() );
	}
	
	
	// Track the mouse
	sel = ::TrackPopupMenu( myMenu, TPM_NONOTIFY | TPM_RETURNCMD, pt.h, pt.v, 0, mWind, NULL );
	::DestroyMenu( myMenu );
	if ( mAtFullScreen )
		::SetCapture( mWind );
	#endif
	
	if ( sel > 0 ) {
		if ( sel == 1 )
			EnableSlideShow();
		else if ( sel >= 1 ) {
			LoadConfig( sel - 1, EgOSUtils::CurTimeMS() - mBaseTimeIndex, true );
			mInSlideShowMode = false;
		}
	}
		
		
	__restorePort
}





void WhiteCap::EnableSlideShow() {

	mInSlideShowMode = true;
	mNextShapeChange = mLastRecordTime;
	mConfigPlayList.Randomize();

}

#if SONIQUE || ENTHOGEN
void WhiteCap::SetFullscreen( bool ) {
}
#else


void WhiteCap::SetFullscreen( bool inFullScreen ) {
	
		
	
	#if SOUNDJAM
	VisHandlerData*	handlerData = (VisHandlerData*) mRefCon;
	if ( handlerData ) {
		if ( mAtFullScreen != inFullScreen ) {
			if ( PlayerSetFullScreen(handlerData->appCookie, handlerData->playerProc, inFullScreen) == noErr)
				mAtFullScreen = inFullScreen;
		}
	}
	#else
	
	
	bool			ok;	
	int				dispID;
	Point			size;

	#if EG_MAC
	GrafPtr	savePort;
	::GetPort( &savePort );
	#endif
	
	if ( inFullScreen && ! mAtFullScreen ) {
		

		#if EG_WIN
		// If the window is maximized, get it back to normal size
		if ( IsZoomed( mWind ) )
			ShowWindow( mWind, SW_RESTORE ); 
		#endif
		
			
		// Update the positon of our win 
		GetWinRect( mWinRectHolder );
		
		// See what device the user user wants fullscreen...
		dispID = ScreenDevice::GetDisplayID( mFullscreenDevice );

		
		#if MACAST
		gPlugInfo.ma -> EnterFullScreen( 'XXXX' /*cPluginAuthor*/, cWhiteCapID );
		#endif	
		
		#if EG_WIN
		::ShowWindow( gWCModule.hwndParent, SW_HIDE | SW_MINIMIZE );
		#else
		::HideWindow( mWind );
		#endif

						
		// Put in pref wheather to try 32 before 16 bit color?
		size = mFullscreenSize;
		ok = mScreen.EnterFullscreen( dispID, size, mFullscreenDepth, mWind, mFullscreenFreq );
		
		if ( ok ) {

			
			Rect r;
			SetRect( &r, 0, 0, size.h, size.v );
			SetPort( mScreen.BeginFrame(), r, true );
			mScreen.EndFrame();
				
			// Changing the port (and the resolution) may change the mouse cords
			EgOSUtils::GetMouse( mLastMousePt );

			// Default: mouse movement will not exit fullscreen mode
			mMouseWillAwaken = false; }	
		else {
			#if MACAST
			gPlugInfo.ma -> ExitFullScreen();
			#endif
					
			#if EG_WIN
			::ShowWindow( gWCModule.hwndParent, SW_SHOWNORMAL );
			#else
			::ShowWindow( mWind );
			#endif
		} 
					
		}
	else if ( ! inFullScreen && mAtFullScreen ) {

		#if MACAST
		gPlugInfo.ma -> ExitFullScreen();
		#endif
		
		// Restore the window
		mScreen.ExitFullscreen();
		

		SetWinPort( mWind, &mWinRectHolder );
		
		
		mLastMousePt.h -= 55;		
		
		#if WINAMP
		::ShowWindow( gWCModule.hwndParent, SW_SHOWNORMAL );
		#endif
	}
	
	#if EG_MAC
	::SetPort( savePort );
	#endif
	
	#endif


	
	#if EG_MAC
	::FlushEvents( 0xFFFF, 0 );
	#endif
	
	// Prevent sleep
	if ( ! mAtFullScreen )
		mLastActiveTime = mLastRecordTime;
}

#endif


void WhiteCap::GetWinRect( Rect& outRect ) {
	
	SetRect( &outRect, 0, 0, 0, 0 );
	
	if ( mWind ) {
	
		#if EG_MAC
		outRect = (**(((CGrafPtr) mWind)->portPixMap)).bounds;
		outRect.left *= -1;
		outRect.top  *= -1;
		outRect.right = outRect.left + mWind -> portRect.right;
		outRect.bottom = outRect.top + mWind -> portRect.bottom;
		#elif WINAMP || ( STANDALONE && EG_WIN )
		RECT wr;
		::GetWindowRect( mWind, &wr );

		outRect.left	= wr.left; 
		outRect.top		= wr.top;
		outRect.right	= wr.right; 
		outRect.bottom	= wr.bottom;
		#endif 
	}

}





void WhiteCap::RecordSample( long inCurTime, float inScale ) {
	float scale = mMagScale * inScale;

	/* Index the time from the start of the WhiteCap.  This is done because we use the original time indexes, we have 
	 very large numbers.  With 32bit ints this is fine, but when they're put in 32 bit floats, we lose critical
	 percision (because each sample's index only varies by 10 or 20 milliseconds.  We could change the virtual machine so 
	 it uses all double floats, but that's a serious slowdown. */
	inCurTime -= mBaseTimeIndex;
	
	// Apply the global magnitude scale to the sample/soundbyte
	for ( int i = 0; i < mNumSampleBins; i++ )
		mSample[ i ] *= scale;
	
	// Smooth the input
	if ( mSmoothFactor > 0 )
		XFloatList::GaussSmooth( mSmoothFactor, mNumSampleBins, mSample );
	
	// We'll need the 'current' time in Draw()
	mLastRecordTime = inCurTime;
	
	// Give the sample data to the window if we're not in capture mode
	if ( ! mCaptureMode ) {
	
		// Loop thru all the panes in the window
		for ( int i = 0; i < mNumWorlds; i++ ) {

			// Tell the world to take a new sound sample (if it wants)
			mWorld[ i ] -> RecordSample( inCurTime, mSample, mNumSampleBins );
		} }
	else {
		mDumpFile.PutByte( CAPTURE_SAMPLE );
		mDumpFile.PutLong( inCurTime );
		mDumpFile.PutLong( mNumSampleBins );
		mDumpFile.PutBlock( mSample, sizeof( float ) * mNumSampleBins );
	}
}





bool WhiteCap::HandleKey( long inChar ) {
	long n;
	bool handled = true;
	
	// See if this keystroke is to be ignored
	if ( ! mHandleKeys || mKeysIgnored.FindNextInstanceOf( 0, inChar ) )
		return false;
	
	switch ( inChar ) {
			
		case 27:  // ESC Key
			SetFullscreen( false );
			break;

		case '%':
			#if EG_MAC
			mCaptureMode = ! mCaptureMode;
			#endif
			
		
			// Create capture data path if we're in capture mode
			if ( mCaptureMode && ! mDumpFile.is_open() ) {
				CEgFileSpec spec;
				UtilStr configName;
				
				CEgIOFile::sCreatorType = cPluginAuthor; 
				spec.AssignPathName( "WhiteCap Capture"  );
				spec.SetType( cWC_CaptureType );
				spec.MakeUnique();
				mDumpFile.open( &spec );
				mDumpFile.PutLong( cWhiteCapID );
				mDumpFile.PutLong( WHITECAP_COMPAT_VERSION );
				mDumpFile.PutLong( mBaseTimeIndex );
				
				// Tell the playback what config we're on now... 
				if ( mConfigs.FetchSpecName( mCurConfigNum, configName ) ) {
					mDumpFile.PutByte( CAPTURE_CONFIG_CHANGE );
					configName.WriteTo( &mDumpFile );
				} }
			else {
				mDumpFile.close();
				Refresh();
			}
			break;
	
		case '[':
			mMagScale /= 1.15;
			break;
			
		case ']':
			mMagScale *= 1.15;
			break;
			
		case '`':
			SetFullscreen( ! IsFullscreen() );
			break;
			
		case '\\':
			if ( mInSlideShowMode )
				mInSlideShowMode = false;
			else
				EnableSlideShow();
			break;
			
		case ',':
		case '.':
			int num = mConfigPlayList.Count();
			n = mConfigPlayList.FindIndexOf( mCurConfigNum );
			if ( inChar == ',' )
				n += num - 2;
			LoadConfig( mConfigPlayList.Fetch( 1 + n % num ), mLastRecordTime, false );
			mInSlideShowMode = false;
			break;
			
		case '(':
			if ( mStepsPerBin > 1 )
				mStepsPerBin--;
			break;
			
		case ')':
			if ( mStepsPerBin < 30 )
				mStepsPerBin++;
			break;
			
		case '{':
			SetNumSampleBins( mNumSampleBins - 4 );
			break;

		case '}':
			SetNumSampleBins( mNumSampleBins + 4 );
			break;		
		
		case 128:
		case 8:
			mKeyBuf.Keep( mKeyBuf.length() - 1 );
			break;		
			
		default:
			if ( inChar >= ' ' && inChar <= '~' ) 
				mKeyBuf.Append( (char) inChar );
			else
				handled = false;			
	}
	
	if ( handled )
		mLastKeyTime = mLastRecordTime;
	
	return handled;
}



void WhiteCap::SetNumSampleBins( long inNumBins ) {

	if ( inNumBins > 0 ) {
		mSample = (float*) mSampleBuf.Dim( sizeof( float ) * inNumBins );
		mNumSampleBins = inNumBins;
		
		for ( int i = 0; i < mNumSampleBins; i++ )
			mSample[ i ] = 0;
	}
}




void WhiteCap::Draw() {
	Rect	r;
	Point	pt;
	long	i, time = mLastRecordTime;
	bool	kybdPress;

	// Do we have any keyboard input to handle?
	if ( mKeyBuf.length() ) {
	
		// Process the kybd buffer only if we haven't gotten any typing in a while
		if ( time > mLastKeyTime + 2500 )
			HandleKybdBuf();
	}

	// Load a random waveshape every so often, and randomize things
	if ( time > mNextShapeChange && mInSlideShowMode )
		LoadNextConfig();	
	
	
	// Don't bother doing mouse or kybd poll if sceeen saver mode is disabled
	if ( mScrnSaverDelay > 0 ) {

		kybdPress = false;

		#if EG_MAC	
		long pollDelay;
		
		// Calc time till next kybd poll (Don't waste valuble time checking the kybd unless we've been idle a while)
		if ( mAtFullScreen )
			pollDelay = 600;
			
		// Don't bother rapildly checking the kybd until we're really close to going into screen saver mode
		else if ( time > mLastActiveTime + mScrnSaverDelay - 90000 )
			pollDelay = ( mLastActiveTime + mScrnSaverDelay - time ) >> 7;
		else
			pollDelay = 10000;
			
		// If it's time to poll the kybd...
		if ( time > mLastKeyPollTime + pollDelay ) {
			::GetKeys( mCurKeys );
			mLastKeyPollTime = time;
			
			// If the keys are pressed, 'move' the mouse (ie, cause the if-block a few lines from now to 'go'
			if ( ( mCurKeys[0] != mPastKeys[0] ) || ( mCurKeys[1] != mPastKeys[1] ) || ( mCurKeys[2] != mPastKeys[2] ) || ( mCurKeys[3] != mPastKeys[3] ) ) {
				kybdPress = true;
				mPastKeys[0] = mCurKeys[0];
				mPastKeys[1] = mCurKeys[1];
				mPastKeys[2] = mCurKeys[2];
				mPastKeys[3] = mCurKeys[3];
			}
		}
		#endif
		
		// Check the mouse pos and record it as active if its been moved.  	
		EgOSUtils::GetMouse( pt );
		if ( pt.h != mLastMousePt.h || pt.v != mLastMousePt.v || kybdPress ) {
			mLastMousePt		= pt;
			mLastActiveTime		= time;
			if ( mAtFullScreen && mMouseWillAwaken )
				SetFullscreen( false ); 
		}
	
		// If we're elligible to enter fullscreen
		if ( ! mAtFullScreen && time > mLastActiveTime + mScrnSaverDelay ) {
			SetFullscreen( true );
			mMouseWillAwaken = true;
		}
	}
	
	// Since we redirect drawing to a API supplied frame buffer, we want to overwrite everything in it
	#if ENTHOGEN
	Refresh();
	#endif
	
	// Save the cur environment
	__setupPort
	
	// If we're fullscreen, follow the API (the screen may need to do something to start the frame)
	if ( mScreen.IsFullscreen() )
		mOutPort = mScreen.BeginFrame();


	if ( ! mCaptureMode ) {
	
		// Loop thru all the panes in the window and the the whitecap wave thingy
		for ( i = 0; i < mNumWorlds; i++ ) {
	
			// Tell the world to update itself to the offscreen draw port
			mWorld[ i ] -> Render( time, mPort, r );
			::SectRect( mWorld[ i ] -> PaneRect(), &r, &r );
	
			// For Sonique and Enthogen, we've already redirected drawing to a given frame buffer (so there's nothing to copy)
			#if ! SONIQUE && ! ENTHOGEN
			Rect destRect = r;
			::OffsetRect( &destRect, mPaneRect.left, mPaneRect.top );
			mPort.CopyBits( mOutPort, &r, &destRect );
			#endif
		}


		#if EG_MAC
		
		// If the option key is down, show the frame rate
		unsigned char km[16];
		::GetKeys( (unsigned long*) km );
		i = 58;	
		if ( ((km[ i >> 3 ] >> (i & 7)) & 1) != 0 ) {
		
			// Calc frame rate, show the current computation for frames
			if ( time - mFrameCountStart >= 1900 ) {
				mFramesPer10Secs = 10000 * mFrameCount / ( time - mFrameCountStart );
				mFrameRate.Assign( mFramesPer10Secs );
				mFrameRate.Insert( mFrameRate.length() - 1, ".", 1 );
				mFrameCountStart = time;
				mFrameCount = 0;
			}
			mFrameCount++;
				
			Rect r = { 0, 0, 20, 40 };
			::EraseRect( &r );
			::MoveTo( 5, 15 );
			::DrawString( mFrameRate.getPasStr() );
		}
		#endif
				
		// See if we want to draw the title...
		bool drawTitle;
		if ( mAtFullScreen )
			drawTitle = ( mShwT_Pref & 0x1 ) != 0;
		else 
			drawTitle = ( mShwT_Pref & 0x2 ) != 0;
			
		if ( drawTitle ) {
			
			// For each pane/world, draw the title string...		
			for ( i = 0; i < mNumWorlds; i++ ) {
				
				// Text drawing this proc is done on the OS port so we have to offset the text to where the WC pane is
				mWorld[ i ] -> DrawConfigName( mOutPort, mPaneRect.left, mPaneRect.top );
			}
		}

	}
	else  {
		#if EG_MAC
		::MoveTo( 5, 20 );
		::DrawString( "\pCapturing..." );
		#endif   
	}

	// If we're fullscreen, follow the API (the screen may need to do something to finish)
	if ( mScreen.IsFullscreen() )
		mScreen.EndFrame();
	
	__restorePort
} 







void WhiteCap::SetWinPort( WindowPtr inWin, const Rect* inRect ) {
	Rect r;
	
	// mDoingSetPortWin == true is a signal that another thread is in SetWinPort()
	if ( mDoingSetPortWin )
		return;
	mDoingSetPortWin = true;
	mWind = inWin;

	if ( inRect )
		r = *inRect;
	
	// If an invalid win rect, fix it, you monkey!
	if ( r.right - r.left < 20  || r.bottom - r.top < 20 || ! inRect ) {
	
		r.top				= mPrefs.GetPref( 'wTop' );
		r.left				= mPrefs.GetPref( 'wLft' );
		r.right				= mPrefs.GetPref( 'wRgt' );
		r.bottom			= mPrefs.GetPref( 'wBot' );
		
		#if EG_MAC
		// Invalidate the window if it's offscreen...
		RgnHandle deskRgn = ::GetGrayRgn();
		::SectRect( &(*deskRgn) -> rgnBBox, &r, &r );
		#endif
	}

	// If no prefs avail (or an older version, use factory rect) or a bad win, hard code a size
	if ( mPrefs.GetPref( 'Vers' ) != WHITECAP_COMPAT_VERSION || r.right - r.left < 20  || r.bottom - r.top < 20 ) {
		r.top = 52;
		r.left = 16;
		r.right = 500;
		r.bottom = 420; 
	}

	long x = r.right - r.left;
	long y = r.bottom - r.top;
	
	
			
	// Resize the window to the rect we were given and redim offscreen draw port...
	if ( inWin ) {
		#if EG_MAC
		x -= x % 4;
		::MoveWindow( inWin, r.left, r.top, true );
		::SizeWindow( inWin, x, y, true ); 
		::ShowWindow( inWin );
		::SetRect( &r, 0, 0, x, y );
		SetPort( inWin, r, false );
		#endif
		
		#if WINAMP || ( EG_WIN && STANDALONE )
		RECT cr;
		::MoveWindow( inWin, r.left, r.top, x, y, false );
		::GetClientRect( inWin, &cr );
		if ( cr.right % 4 > 0 )
			::MoveWindow( inWin, r.left, r.top, x - cr.right % 4, y, true );
		::GetClientRect( inWin, &cr );
		::SetRect( &r, 0, 0, cr.right, cr.bottom );		
		SetPort( NULL, r, false );
		#endif
	}
	
	#if SONIQUE
	::SetRect( &r, 0, 0, inRect -> right, inRect -> bottom );		
	SetPort( NULL, r, false );				
	#endif	

	// Signal that this thread is done with SetPortWin()
	mDoingSetPortWin = false;
}





void WhiteCap::SetPort( GrafPtr inPort, const Rect& inRect, bool inFullScreen ) {
	long x = inRect.right - inRect.left;
	long y = inRect.bottom - inRect.top;


	
	mOutPort = inPort;
	#if ! SONIQUE && ! ENTHOGEN
	mPort.Init( x, y );
	#endif	
	mAtFullScreen = inFullScreen;

	// mPaneRect is the rect within inPort the plugin frame occupies
	mPaneRect = inRect;
		
	
	// If we're brand spankin new, show at least 1 config, you monkey
	if ( mNumWorlds == 0 ) {
		LoadConfig( mCurConfigNum, mLastRecordTime, false );
		
		// Since we're new, do the first slide quickly
		mNextShapeChange = mLastRecordTime + 6000;
	}
		
	for ( int i = 0; i < mNumWorlds; i++ ) {
		mWorld[ i ] -> ExpireSamples();
	}
		
	ResizeWorlds();
	Refresh();
}






#define ___max( a, b ) (( (a) > (b) ) ? (a) : (b))
#define ___maxSide( r ) (___max( r.right - r.left, r.bottom - r.top ))


	
void WhiteCap::ResizeWorlds() {
	Rect r;
	long i, j, bestLen, cutLen, best;

	
	// Resize the panes in this window:  We made our PixPort the size of the rect we're drawing in, so all
	// WhiteCap cords within worlds are relative to the corner of where the PixPort origin is.  That is,
	// the port cords of point (3,4) in a WhiteCap world are (mPaneRect.left+3, mPaneRect.top+4).
	r = mPaneRect;
	::OffsetRect( &r, -r.left, -r.top );
	mWorld[ 0 ] -> SetPaneRect( r );
	for ( i = 1; i < mNumWorlds; i++ ) {
		
		// Find the best world to cut in half
		for ( bestLen = -1, j = 0; j < i; j++ ) {
			cutLen = ___maxSide( (*mWorld[ j ] -> PaneRect()) );
			if ( cutLen >= bestLen ) {
				bestLen = cutLen;
				best = j;
			}
		}
		
		// Change the pane rect of both the unplaced world and the world that's about to get cut in half
		r = *(mWorld[ best ] -> PaneRect());
		if ( r.right - r.left == bestLen ) {
			r.right -= bestLen / 2;
			mWorld[ best ] -> SetPaneRect( r );
			OffsetRect( &r,  bestLen / 2,  0 );
			mWorld[ i ] -> SetPaneRect( r ); }
		else {
			r.bottom -= bestLen / 2;
			mWorld[ best ] -> SetPaneRect( r );
			OffsetRect( &r,  0,  bestLen / 2 );
			mWorld[ i ] -> SetPaneRect( r ); 
		}				
	}
}











void WhiteCap::Refresh() {
	
	for ( int i = 0; i < mNumWorlds; i++ ) {
		mWorld[ i ] -> Refresh();		
	}
}





void WhiteCap::LoadNextConfig() {
	int i;
	
	// Load the next config in the (randomized) config list...
	i = mConfigPlayList.FindIndexOf( mCurConfigNum );
	
	// Make a new play list if we've reached the end of the list...
	if ( i >= mConfigPlayList.Count() ) {
		mConfigPlayList.Randomize();
		i = 0;
	}
	LoadConfig( mConfigPlayList.Fetch( i + 1 ), mLastRecordTime, true );
}



void WhiteCap::LoadConfig( int inConfigNum, long inCurTime, bool inAllowMorph ) {
	const CEgFileSpec* configSpec;
	int i, exists;
	long oldNumWorlds = mNumWorlds;
	long transitionTime;
		
	// See how long we're gonna let the existing config transform into the oncoming config
	if ( inAllowMorph )
		transitionTime = EgOSUtils::Rnd( mTransitionLo * 1000, mTransitionHi * 1000 );
	else
		transitionTime = 1;
		
	mNumWorlds = 0;
	mCurConfigNum = -1;
	
	// Don't bother initiating a config change if we're already running that config
	if ( inConfigNum == mCurConfigNum )
		return;
		
	// Fetch the spec for our config file or folder
	configSpec = mConfigs.FetchSpec( inConfigNum );
	
	if ( configSpec ) {
 
 		// If we have a folder or file...
 		exists = configSpec -> Exists();
		if ( exists == 2 ) {
			CEgFileSpec spec;
			
			bool startOver = true;
			while ( EgOSUtils::GetNextFile( *configSpec, spec, startOver, false ) ) {
				if ( mNumWorlds >= oldNumWorlds )
					mWorld[ mNumWorlds ] = new WhiteCapWorld;
				mWorld[ mNumWorlds ] -> Init( &spec, inCurTime, transitionTime, mNumSampleBins );
				mNumWorlds++;
				startOver = false;
			}
		}
		
		// Know what to put a check mark next to in the popup menu
		mCurConfigNum = inConfigNum; 
	}
	
	// If we're in capture, signal a config change...
	if ( mCaptureMode ) {
		UtilStr name;
		if ( mConfigs.FetchSpecName( mCurConfigNum, name ) ) {
			mDumpFile.PutByte( CAPTURE_CONFIG_MORPH );
			name.WriteTo( &mDumpFile );
		}
	}

	
	// If it's only one config file or there's no spec specifed
	if ( mNumWorlds == 0 ) {
		mNumWorlds = 1;
		if ( mNumWorlds > oldNumWorlds )
			mWorld[ 0 ] = new WhiteCapWorld;
		mWorld[ 0 ] -> Init( configSpec, inCurTime, transitionTime, mNumSampleBins );
	}
	
	for ( i = mNumWorlds; i < oldNumWorlds; i++ )
		delete mWorld[ i ];

	// Set the time for when we load a random config automatically
	mNextShapeChange = mLastRecordTime + mSlideShowInterval + transitionTime;
	
	// If the cursor was spun, init it back to the ptr
	EgOSUtils::ResetCursor();

	// Make sure all WhiteCap's configs live happily inside the allowed draw area
	ResizeWorlds();

	// Very annoying:  in Win32, the call to GetWindowRect() is returning garbage in ~WhiteCap(), so just 
	// forget about saving the window position for now in windows :_(
	#if EG_WIN
	if ( ! mAtFullScreen )
		GetWinRect( mWinRectHolder );
	#endif

	if ( mAtFullScreen )
		EgOSUtils::HideCursor();
}




void WhiteCap::BuildConfigList( CEgFileSpec& inPluginsFolder ) {
	CEgFileSpec spec;
	bool startOver = true;
	int i;
		
	while ( EgOSUtils::GetNextFile( inPluginsFolder, spec, startOver, false ) ) {
		mConfigs.AddCopy( spec );
		startOver = false;
	}
	
	startOver = true;
	while ( EgOSUtils::GetNextFile( inPluginsFolder, spec, startOver, true ) ) {
		mConfigs.AddCopy( spec );
		startOver = false;
	}
	
	mCurConfigNum = 1;

	// Always have a item called (Default) in the config list
	if ( mAddDefaultConfig ) {
		UtilStr		def( "(Default)" );
		spec.Assign( inPluginsFolder );
		if ( ! mConfigs.Lookup( def ) ) {
			spec.Rename( def );
			mConfigs.AddCopy( spec );
		}
		mCurConfigNum = mConfigs.Lookup( def );
	}

	// Build a random config 'play' list
	mConfigPlayList.RemoveAll();
	for ( i = mConfigs.Count(); i > 0; i-- ) 
		mConfigPlayList.Add( i );
	mConfigPlayList.Randomize();	
}




void WhiteCap::HandleKybdBuf() {
	long i;
		

	// Look for a config that best matches what was typed.  Use exact match if user didn't type a lot
	if ( mKeyBuf.length() >= 4 )
		i = mConfigs.FetchBestMatch( mKeyBuf ); 
	else
		i = mConfigs.Lookup( mKeyBuf );
		
	if ( i > 0 ) {
		LoadConfig( i, mLastRecordTime, true );
		
		// Changing a config via kybd disable slide show mode
		mInSlideShowMode = false;
	}
	
	mKeyBuf.Wipe();
}








#if MACAST || ( EG_MAC && STANDALONE ) || ENTHOGEN


long WhiteCap::HandleEvt( WindowPtr inWin, EventRecord& evt ) {
 	static long sLastWhen = 0;
	long 	curTime = ::TickCount();
	Point inPt;
	long	handled = 1;
		
	switch ( evt.what ) {

						
		case updateEvt:	
			::BeginUpdate( inWin );
			Refresh();
			::EndUpdate( inWin );
			break;
			
		
		case mouseDown:	
			inPt = evt.where;
			if ( PtInTitle( inPt ) || ( evt.modifiers & cmdKey ) )
				SelectConfig();
			else if ( IsFullscreen() )
				SetFullscreen( false );
			else if ( curTime - sLastWhen < ::GetDblTime() ) 
				SetFullscreen( true );
			else if ( ! IsFullscreen() ) {
				Rect r, growBox, lim;
				GetWinRect( r );
				growBox = r;
				growBox.left = r.right - 30;
				growBox.top = r.bottom - 30;
				if ( ::PtInRect( inPt, &growBox ) && inWin ) {
					lim.top = lim.left = 30;
					lim.bottom = lim.right = 9000;
					unsigned long newSize = ::GrowWindow( inWin, inPt, &lim );
					::SizeWindow( inWin, newSize & 0xFFFF, newSize >> 16, false );
					GetWinRect( r );
					SetWinPort( inWin, &r );	}	// SizeWindow changes the size of the win, but gWC doesn't know that
				else if ( BorderlessWindow() && ::Button() && inWin ) {
					lim.top = lim.left = 0;
					lim.bottom = lim.right = 9000;
					::DragWindow( inWin, inPt, &lim );
				}
				sLastWhen = curTime; }
			else
				handled = 0;
			break;
			
		default:
			handled = 0;
	}
	
	return handled;
}



#endif





#if STANDALONE && EG_MAC



bool WhiteCap::PlaybackSample( CEgIStream* inStream, long* outTimeIndex ) {
	long i, captureCode, timeIndex;
	UtilStr str;
	bool done = false;
	
	

	while ( inStream -> noErr() && ! done ) {
	
		captureCode = inStream -> GetByte();
		
		switch ( captureCode ) {
		
			case CAPTURE_SAMPLE:
			
				// Mimic a legit sound sample being recorded.  
				// The ( + mBaseTimeIndex ) term is a trick that negates RecordSample()'s (- mBaseTimeIndex) 
				timeIndex = inStream -> GetLong() + mBaseTimeIndex;
				if ( outTimeIndex )
					*outTimeIndex = timeIndex;
				SetNumSampleBins( inStream -> GetLong() );
				inStream -> GetBlock( mSample, sizeof( float ) * mNumSampleBins );
				done = true;
				break;
			
			case CAPTURE_CONFIG_CHANGE:
			case CAPTURE_CONFIG_MORPH:
			
				str.ReadFrom( inStream );
				i = mConfigs.FetchBestMatch( str ); 
				LoadConfig( i, mLastRecordTime, captureCode == CAPTURE_CONFIG_MORPH );
				mInSlideShowMode = false;
				break;
		}
	}
	
	
	return done;
}






// INCLUDES
#include <Traps.h>
#include <Gestalt.h>
#include <FixMath.h>
//#include <Fonts.h>

#include <QuicktimeComponents.h>
#include <MoviesFormat.h>
#include <Movies.h>


// DEFINES
#define	BailNil(n)		if (!n) goto bail;
#define	BailError(n)	if (n) goto bail;


// FUNCTION PROTOTYPES
pascal short
DefaultsHookProc(DialogPtr theDialog, short itemHit, void *params, long refcon);


void WhiteCap::ExportQT( CEgIFile* inStream ) {

	OSErr		result;
	short		firstPass = true;
	
	Point		where;
	SFReply		dstFile;
	short		movieRefNum = 0;
	
	ComponentInstance ci;
	
	//Movie		srcMovie = nil;
	//Rect		srcRect;
	//GWorldPtr	srcGWorld = nil;
	Movie		dstMovie = nil;
	Track		dstTrack;
	Media		dstMedia;
	long		timeIndex;
	//TimeValue	curMovieTime;
	
//	CWindowPtr	progressWindow = nil;
//	WindowRef	progressWindow = nil;
	
	
	ImageDescription **idh;
	SCTemporalSettings ts;
	//SCDataRateSettings ds;
	


	//---------------------------------------------------------------------------------------------
	//	Open the Standard Compression component and adjust it to meet our needs.
	//---------------------------------------------------------------------------------------------
	
	ci = OpenDefaultComponent(StandardCompressionType,StandardCompressionSubType);
	BailNil(ci);
	
	//	Turn off best depth option in the compression dialog.
	//	Because all of our buffering is done at 32-bits, regardless
	//	of the depth of the source data, best depth isn't very useful
	//	as it will always choose 32-bit.
	//
	//	A more ambitious approach would be to loop through each of the
	//	video sample descriptions in each of the video tracks looking
	//	for the deepest depth, and using that for the best depth.
	//	Better yet, we could find out which compressors were used
	//	and set one of those as the default in the compression dialog.
	
	{
		long flags;
		
		SCGetInfo(ci,scPreferenceFlagsType,&flags);
		flags &= ~scShowBestDepth;
		SCSetInfo(ci,scPreferenceFlagsType,&flags);
	}
	
	//	Because we are recompressing a movie that may have a variable frame rate,
	//	we want to allow the user to leave the frame rate text field blank.  We
	//	can then preserve the frame durations of the source movie.  If they enter
	//	a number we will resample the movie at a new frame rate.  If we don't clear
	//	this flag, the compression dialog will not allow zero in the frame rate field.
	//
	//	NOTE: Setting this flag could have been done above when we cleared the
	//	scShowBestDepth flag.  It is done separately for clarity.
	
	{
		long flags;
		
		SCGetInfo(ci,scPreferenceFlagsType,&flags);
		flags |= scAllowZeroFrameRate;
		SCSetInfo(ci,scPreferenceFlagsType,&flags);
	}
	

	// Make a preview image for the SC setting dialog...
	{
		long pos, startTimeIndex, ok;
	
		// Start WhiteCap by a few frames to get a decent poster image
		pos = inStream -> tell();
		ok = PlaybackSample( inStream, &startTimeIndex );
		timeIndex = startTimeIndex;
		while ( timeIndex - startTimeIndex < 1500 && ok ) {
			RecordSample( timeIndex, 1 );
			ok = PlaybackSample( inStream, &timeIndex );
		}
		inStream -> seek( pos );


		//	Use the gworld image of the movie poster frame as a compression dialog test image.
		//	Pass nil for srcRect to use the entire image, and pass zero to use the default display method.

		if ( ok ) {
			Refresh();
			Draw();
			result = SCSetTestImagePixMap(ci, mPort.GetPixMap(),nil,0);
			BailError(result);
		}
		
		for ( int i = 0; i < mNumWorlds; i++ )
			mWorld[ i ] -> ExpireSamples();		
	}
/*	
	{
		//PicHandle	ph = GetMoviePosterPict(srcMovie);
		CGrafPtr	savePort;
		GDHandle	saveDevice;
		
		//result = GetMoviesError();
srcRect.left = 0;
srcRect.top = 0;
srcRect.bottom = 100;
srcRect.right = 200;		// change this

		//GetMovieBox(srcMovie,&srcRect);
		result = NewGWorld(&srcGWorld,32,&srcRect,nil,nil,0);
		//BailError(result);

	}
*/	
	//---------------------------------------------------------------------------------------------
	//	Set compression dialog extend procs for the "Defaults" button.
	//	We wait until now so we can pass the srcGWorld pixmap in as a refcon.
	//---------------------------------------------------------------------------------------------

	{
		SCExtendedProcs xprocs;
		
		//	Because the compression dialog is movable modal, a filter proc
		//	to handle update events for application windows is required.
		//	Since our example application is completely modal, and we have
		//	no other windows to update, we don't have a filter proc.
		
		xprocs.filterProc = nil;
		
		//	Proc to handle custom button click.
		
		xprocs.hookProc = NewSCModalHookProc(DefaultsHookProc);
		
		//	Any information useful to the extended procs can be put
		//	in the refcon.  For an application that has globals, a refcon
		//	of the current A5 is handy for getting at those globals in
		//	the extended procs.  In our case, we put the srcGWorld pixmap
		//	in the refcon so we can set defaults on it in our hook proc.
		
		xprocs.refcon = (long) ( mPort.GetPixMap() );
		
		//	Set the custom button name.
		
		BlockMove("\pDefaults",xprocs.customName,9);
		
		//	Tell the compression dialog about the extended procs.
		
		SCSetInfo(ci,scExtendedProcsType,&xprocs);
	}
	
	
	//---------------------------------------------------------------------------------------------
	//	Set up some default settings for the compression dialog if needed, and ask
	//	for compression settings from the user.
	//---------------------------------------------------------------------------------------------
	
	if (firstPass) {
		result = SCDefaultPixMapSettings(ci, mPort.GetPixMap(),true);
		BailError(result);
	}
	
	result = SCGetInfo(ci,scTemporalSettingsType,&ts);
	BailError(result);
	
	//	The first time through, clear out the default frame rate chosen by Standard Compression.
	//	We know that a frame rate of 0 means use the rate of the source movie.  It probably should
	//	have been smart enough to figure that out because we set the scAllowZeroFrameRate flag.
	//	Oh well.
	
	if (firstPass) {
		ts.frameRate = 0;
		SCSetInfo(ci,scTemporalSettingsType,&ts);
	}
	
	//	Get compression settings from the user.  The first time through this loop,
	//	we choose default compression settings for the test image we set above.
	//	On subsequent passes, the settings previously chosen by the user will be the defaults.
	
	result = SCRequestSequenceSettings(ci);
	if (result == scUserCancelled) {
		// deal with user cancelling.
	}
	BailError(result);
	
	//	Get a copy of the temporal settings the user entered.  We'll need them for
	//	some of our calculations.  In a simpler application we'd never have to look at them.
	
	result = SCGetInfo(ci,scTemporalSettingsType,&ts);
	BailError(result);
	
	



	//---------------------------------------------------------------------------------------------
	//	If the user said they want to resample the frame rate of the movie,
	//	(by entering a non-zero value in the frame rate field) calculate
	//	the number of frames and duration for the new movie.
	//---------------------------------------------------------------------------------------------
	
/*
	if (ts.frameRate) {
		long	dur = GetMovieDuration(srcMovie);
		long	timescale = GetMovieTimeScale(srcMovie);
		float	f = (float)dur * ts.frameRate;
		frameCount = f / timescale / 65536;
		if (frameCount == 0)
			frameCount = 1;
	}*/
	
	
	//---------------------------------------------------------------------------------------------
	//	Ask the user for the name of the new movie file.
	//	We'll be lazy here and just use "Untitled".  A real app would
	//	base it on the name of the source movie and check if the user
	//	tried to enter a name the same as the source movie.
	//
	//	Note we use the SCPositionDialog call to get a good position
	//	point for the SFPutFile.  -3999 is the resource I.D. for
	//	SFPutFile dialog.
	//---------------------------------------------------------------------------------------------
	
	where.h = where.v = -2;
	SCPositionDialog(ci,-3999,&where);
	SFPutFile(where,"\pSave new movie file as:","\pUntitled",nil,&dstFile);
	if (!dstFile.good)
		goto bail;			// deal with user cancelling
	
	
	//---------------------------------------------------------------------------------------------
	//	Open a progress window to frames as they are compressed.  Use Standard
	//	Compression utility routines to position the window.
	//---------------------------------------------------------------------------------------------
	
/*	{
		Rect r = srcRect;
		where.h = where.v = -2;
		result = SCPositionRect(ci,&r,&where);
		progressWindow = NewCWindow(0,&r,dstFile.fName,true,0,(WindowPtr)-1,false,0);
	}
	*/
	
	//---------------------------------------------------------------------------------------------
	//	Create the new movie file and prepare it for edits.
	//---------------------------------------------------------------------------------------------
	
	{
		FSSpec			theFSSpec;
		MatrixRecord	matrix;
		
		//	Create an FSSpec for the destination movie.
		
		result = FSMakeFSSpec(dstFile.vRefNum,0,dstFile.fName,&theFSSpec);
		if (result != fnfErr)		// ignore a file not found error
			BailError(result);
		
		//	Using the FSSpec create a movie file for the destination movie.
		
		result = CreateMovieFile(&theFSSpec,'TVOD',0,createMovieFileDeleteCurFile /*| createMovieFileDontCreateResFile*/, &movieRefNum,&dstMovie);
		BailError(result);
		
		//	Create a new video movie track with the same dimensions as the entire source movie.
		
		dstTrack = NewMovieTrack(dstMovie,
				(long)(mPaneRect.right - mPaneRect.left) << 16,
				(long)(mPaneRect.bottom - mPaneRect.top) << 16,0);
		
		//	Create a media for the new track with the same time scale as the
		//	source movie.  Because the time scales are the same, we don't have
		//	to do any time scale conversions.
		
		dstMedia = NewTrackMedia(dstTrack,VIDEO_TYPE,1000,0,0);
		BailError(GetMoviesError());
		
		//	Copy the user data and settings from the source to the dest movie.
		//	These settings include information like user data.
		
		//CopyMovieSettings(srcMovie,dstMovie);
		
		//	Set movie matrix to identity and clear the movie clip region
		//	because conversion process transforms and composites all video
		//	tracks into one untransformed video track.
		
		SetIdentityMatrix(&matrix);
		SetMovieMatrix(dstMovie,&matrix);
		SetMovieClipRgn(dstMovie,nil);
		
		//	Prepare for adding frames to the movie.
		//	Make sure you do this.  It's very easy to forget about.
		
		result = BeginMediaEdits(dstMedia);
		BailError(result);
	}
	
	
	//---------------------------------------------------------------------------------------------
	//	Start a compression sequence using the parameters chosen by the user.
	//	Pass nil for the source rect to use the entire image.  An image description
	//	handle will be returned in idh.  Nil could be passed for that if we
	//	didn't need it for our progress window display.  We do not have dispose
	//	the image description handle.  It is disposed for use by SCCompressSequenceEnd.
	//---------------------------------------------------------------------------------------------
		
	// Erase the current pixmap and draw the first frame
	Refresh();
	PlaybackSample( inStream, &timeIndex );
	RecordSample( timeIndex, 1 );
	Draw();

	result = SCCompressSequenceBegin(ci,mPort.GetPixMap(),nil,&idh);
	BailError(result);

	
	//	Loop through all of the interesting times we counted above.
	
	{

		short			syncFlag;
		TimeValue		duration;
		long			dataSize;
		Handle			compressedData;
		long			minDuration, prevTimeIndex = timeIndex;
		bool			ok = true;

		
		if ( ts.frameRate )
			minDuration = 1000.0 * ( (float) 0x10000 ) / ( (float) ts.frameRate );
		else
			minDuration = 8;
			
		// MAIN FRAME LOOP
		while ( ok ) {
	
			while ( timeIndex - prevTimeIndex < minDuration && ok )
				ok = PlaybackSample( inStream, &timeIndex );
				
			if ( ! ok )  break;
					
			// times on WC are in MS
			duration = ( timeIndex - prevTimeIndex );
			prevTimeIndex = timeIndex;
	
			// Tell WhiteCap to render this sample
			RecordSample( timeIndex, 1 );
			Draw();
			EgOSUtils::SpinCursor();

			
			//	If data rate constraining is being done, tell Standard Compression the
			//	duration of the current frame in milliseconds.  We only need to do this
			//	if the frames have variable durations.
			// 	Without this, the compressor can't limit the data rate.
			
			{
				SCDataRateSettings ds;
				if (!SCGetInfo(ci,scDataRateSettingsType,&ds)) {
					ds.frameDuration = duration;
					SCSetInfo(ci,scDataRateSettingsType,&ds);
				}
			}
			
			//	Compress the frame.  compressedData will hold a handle to the newly compressed
			//	image data.  dataSize is the size of the compressed data, which will usually be
			//	different than the size of the compressedData handle.  syncFlag is a value that
			//	can be passed directly to AddMediaSample which indicates whether or not the frame
			//	is a key frame.  Note that we do not have to dispose of the compressedData handle.
			//	It will be dispose for us when we call SCCompressSequenceEnd.
			
			result = SCCompressSequenceFrame(ci,mPort.GetPixMap(),nil,&compressedData,&dataSize,&syncFlag);
			BailError(result);
			
			//	Append the compressed image data to the media.
			
			result = AddMediaSample(dstMedia,compressedData,0,dataSize,duration,
					(SampleDescriptionHandle)idh,1,syncFlag,nil);
			BailError(result);
		}
	}
	
	
	// If any prefs were changed, don't save the changes
	mPrefs.SetNotDirty();

	
	//	Close the compression sequence.  This will dispose of the image description
	//	and compressed data handles allocated by SCCompressSequenceBegin.
	
	SCCompressSequenceEnd(ci);
	
	


	//---------------------------------------------------------------------------------------------
	//	Now that we're finished compressing video data, make that data part of our movie. 
	//---------------------------------------------------------------------------------------------
	
	if (dstTrack) {
		short resID = 128;
		
		//	End changes to the media.
		
		result = EndMediaEdits(dstMedia);
		BailError(result);
		
		//	Insert the newly created media into the newly created track at
		//	the begining of the track and lasting for the entire duration of
		//	the media.  The media rate is 1.0 for normal playback rate.
		
		InsertMediaIntoTrack(dstTrack,0,0,GetMediaDuration(dstMedia),fixed1);
		
		//	Add the movie resource to the dst movie file.
		
		result = AddMovieResource(dstMovie,movieRefNum,&resID,"\pWhiteCap Movie");
		//result = AddMovieResource(dstMovie,movieRefNum, (short*) movieInDataForkResID,"\pWhiteCap Movie");
		BailError(result);


		//---------------------------------------------------------------------------------------------
		//	Next we flatten the movie.  The movie file we just created has all of
		//	the video data at the beginning and all of the sound data at the end.
		//	This unnatural situation may cause the movie to play poorly.  By flattening
		//	the movie, we are reinterleaving all of the tracks in a way optimal for playback.
		//	Because we cannot flatten a movie in place, we create a temp file, flatten to
		//	that file and if successful, delete the first file and rename the temp file to
		//	be the real file.  For large files this takes some time.
		//
		//	Once again, a real application would choose a temp file name that doesn't
		//	possibly conflict with other file names, as well as delete the temp file
		//	if the flatten fails.
		//---------------------------------------------------------------------------------------------
		{
			FSSpec	theFSSpec;
			short	resID = 128;
			dstFile.fName[++dstFile.fName[0]] = '@';
			result = FSMakeFSSpec(dstFile.vRefNum,0,dstFile.fName,&theFSSpec);
			if (result != fnfErr)
				BailError(result);
			FlattenMovie(dstMovie,0,&theFSSpec,'TVOD',-1,createMovieFileDeleteCurFile,&resID,"\p");
			result = GetMoviesError();
			CloseMovieFile(movieRefNum);
			DisposeMovie(dstMovie);
			dstTrack = nil;
			if (!result) {
				Str255 tempStr;
				BlockMove(dstFile.fName,tempStr,256);
				tempStr[0]--;
				result = FSDelete(tempStr,dstFile.vRefNum);
				result = Rename(dstFile.fName,dstFile.vRefNum,tempStr);
			}
		}
	}
	
	
	//	Just to be overly safe, we clear our the test image
	//	because we just disposed the pixmap it depends on.

	SCSetTestImagePixMap(ci,nil,nil,0);

	
	//	Close the Standard Compression component.
	
	CloseComponent(ci);
bail:;
	EgOSUtils::ShowCursor();
}


//	Handle clicks in the "Defaults" custom button.

pascal short
DefaultsHookProc(DialogPtr theDialog, short itemHit, void *params, long refcon)
{
	#pragma unused(theDialog)
	if (itemHit == scCustomItem) {
		SCTemporalSettings ts;
		
		//	Set defaults for our test image (passed in refcon).
		
		SCDefaultPixMapSettings( (ComponentInstanceRecord*) params,(PixMapHandle)refcon, true );
		
		//	Once again, don't use the default frame rate chosen
		//	by Standard Compression.  Clear it out to zero.
		
		SCGetInfo( (ComponentInstanceRecord*) params,scTemporalSettingsType,&ts);
		ts.frameRate = 0;
		SCSetInfo( (ComponentInstanceRecord*) params,scTemporalSettingsType,&ts);
	}
	return (itemHit);
}



#endif

