/*====================================================================================
 
	Sonique Visual Plugin API

        (c) 1999 Sonique Team, edited by Rajesh Singh (shock@ihug.com.au)

====================================================================================*/

#include <windows.h>

#include "WhiteCap.h"
#include "EgOSUtils.h"

#include "Sonique_vis.h"



/*====================================================================================
	Definitions of the functions for Plug-In (don't change)
====================================================================================*/
void initialize(void);
BOOL render(unsigned long* Video, int width, int height, int pitch, VisData* pVD);
BOOL savesettings(char* FileName);
BOOL opensettings(char* FileName);


char szAppName[] = "WhiteCap"; // Our window class, etc


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);


/*====================================================================================
	The actual details for your plugin

	1. Reserved (leave as NULL)
	2. Plug-In Name (char*, the name you want displayed when your plug-in is playing)
	3. Options (VI_WAVEFORM - if you need the waveform PCM data
	            VI_SPECTRUM - if you need the spectrum FFT data
				SONIQUEVISPROC - if you want user defined blurs to effect your plugin)
	4. Name of the Initialize function (leave as is)
	5. Name of the Render function (leave as is)
	6. Name of the SaveSettings function (leave as is)
	7. Name of the OpenSettings function (leave as is)
====================================================================================*/
VisInfo plugin = 
{
	0,
	szAppName,
	VI_SPECTRUM,
	&initialize,
	&render,
	&savesettings,
	&opensettings,
	NULL,
	NULL,
	NULL
};

static WhiteCap*	gWC = NULL;
static Rect			gRect = { -1, -1, -2, -2 };


/*====================================================================================
	Definition of the DLL export (don't change)
====================================================================================*/
DLLEXPORT VisInfo* QueryModule(void)
{
	return &plugin;
}

/*====================================================================================
	Definition of the Initialize function

		fill in with whatever you need to initialize, this gets called before the 
		plug-in starts.

		you can leave it empty (as is).
====================================================================================*/
void initialize()
{

	
	EgOSUtils::Initialize( NULL );

	CEgFileSpec folder;
	folder.AssignFolder( "\\Vis\\WhiteCap\\WhiteCap Configs" );

	ScreenDevice::sMinDepth = 32;

	gWC = new WhiteCap( folder );	
	
	// Invalidate the window rect...
	gRect.right = -2;

/*	
	gWC_Win = CreateWindowEx(
			WS_EX_OVERLAPPEDWINDOW,			// these exstyles put a nice small frame, 
												// but also a button in the taskbar
			szAppName,							// our window class name
			"blah",				// use description for a window title
			WS_OVERLAPPEDWINDOW,				// make the window visible with a close button
			100,100,100,100,						// screen position
			NULL,				// parent window (winamp main window)
			NULL,								// no menu
			NULL,	0);			// hInstance of DLL
	



	gWC -> SetWinPort( gWC_Win ); */
//	::ShowWindow( hMainWnd, SW_SHOWNORMAL );

	
}

/*
BOOL deinit() {

	delete gWC;
	gWC = NULL;
	EgOSUtils::ShowMsg( "Go Deinit" );
	

	//::DestroyWindow( gWC_Win ); // delete our window
	//::UnregisterClass( szAppName, this_mod->hDllInstance); // unregister window class


	EgOSUtils::Shutdown();
	
	return TRUE;
}
*/

/*====================================================================================
	Definition of the Render function

		this is where you place your visual code, 
		
			Render should draw a whole frame into the 32 bit ABGR buffer (for blur, smoke 
			and zoom to work it should, rather than replacing the video data, add to it 
			using bitwise or, saturated add, or	alpha blend). Make sure not to exceed the 
			boundaries given. 'Pitch' specifies	the distance, in pixels, between the start 
			of each line. If you have a pointer	at the start of a line, Pointer+Pitch is 
			the start of the next line and Pointer-Pitch is the start of the previous.

		this is the only function that you have to have something in.

			Called for each frame. Pitch is in pixels and can be negative.
			Render like this:
			for (y = 0; y < height; y++)
			{
			   for (x = 0; x < width; x++)
			      Video[x] = <pixel value>;
			  Video += pitch;
			}
						OR
			void PutPixel(int x, int y, unsigned long Pixel)
			{
			   _ASSERT( x >= 0 && x < width && y >= 0 && y < height );
			  Video[y*pitch+x] = Pixel;
			}

		return TRUE when you've finished draw to the buffer.



====================================================================================*/

#define SKIP_BINS 32

BOOL render(unsigned long* Video, int inWidth, int inHeight, int pitch, VisData* pVD)
{
	
	if ( ! gWC )
		initialize();


	// Determine if we've resized
	if ( inWidth != gRect.right || inHeight != gRect.bottom ) {
		gRect.left		= 0;
		gRect.top		= 0;
		gRect.right		= inWidth;
		gRect.bottom	= inHeight;
		
		gWC -> SetWinPort( NULL, &gRect );
	}

	// Tell WC where to draw the next frame	
	gWC -> RedirectFrame( (char*) Video, 4 * pitch, 32, inWidth, inHeight - 1 );

	
	long sum, idx;
	float*  sample = gWC -> mSample;
	int i, j;
	

	float binVal;
	long stepsPerBin = gWC -> mStepsPerBin, numBins = gWC -> mNumSampleBins;
	
	// Restrict case when we have too many WC bins
	if ( numBins > 255 )
		numBins = 255;
	
	// Limit number of bins if there's not enough mp3 player sample to go around
	else if ( numBins * stepsPerBin > (255 - SKIP_BINS) )
		numBins = (255 - SKIP_BINS) / stepsPerBin;


	idx = SKIP_BINS;
	for ( i = 0; i < numBins; i++ ) {
		sum = 0;
		for ( j = 0; j < stepsPerBin; j++, idx++ ) {
			sum += ((unsigned) pVD->Spectrum[0][idx]);
			sum += ((unsigned) pVD->Spectrum[1][idx]);
		}
		binVal = ((float) sum) * (0.81 + (float) i / 24);
		sample[ i ] = log( .00645 * binVal + 1.7 ) - log( 1.7 );
	}
		
	sample[ 0 ] *= 0.35;
	sample[ 1 ] *= 0.75;
	
	// If we didn't have enough bins to fill the WC sample, zero out the rest
	for ( i = numBins; i < gWC -> mNumSampleBins; i++ )
		sample[ i ] = 0;

	gWC -> RecordSample( EgOSUtils::CurTimeMS(), 2.5 );

	gWC -> Draw();

	return TRUE;
}

/*====================================================================================
	Definition of the SaveSettings function

		to save the settings of anything you want regarding your plugin.

		use WritePrivateProfileString to save the settings.
			ie. WritePrivateProfileString("my plugin", "brightness", "3", FileName);

		return TRUE if the saving worked or FALSE if it failed.

		you can leave it empty (as is).
====================================================================================*/
BOOL savesettings(char* )
{

	if ( gWC ) {
		delete gWC;
		gWC = NULL;


		EgOSUtils::Shutdown();
	}

	return TRUE;
}

/*====================================================================================
	Definition of the OpenSettings function

		to open the settings of anything you want regarding your plugin.

		use GetPrivateProfileString to save the settings.
			ie. char BrightnessBuffer[256];
				GetPrivateProfileString("my plugin", "brightness", "3", BrightnessBuffer, sizeof(BrightnessBuffer), FileName);

		return TRUE if the opening worked or FALSE if it failed.

		you can leave it empty (as is).
====================================================================================*/
BOOL opensettings(char*)
{
	return TRUE;
}





