
#include <windows.h>

#include "WhiteCap.h"
#include "EgOSUtils.h"
#include "RectUtils.h"
#include "CEgFileSpec.h"

#include "Winamp_VIS.h"
#include "SoundInputPort.h"

#include "Standalone.h"



extern winampVisModule	gWCModule;
extern HWND				gWC_Win;
extern WhiteCap*			gWC;

bool	gQuitting = false;

#define __WC_MutexName  "WhiteCap Event Mutex"



DWORD WINAPI WCThread( LPVOID lpParam );
DWORD WINAPI WCThread( LPVOID  ) {

	SoundInput  soundPort;
	float		binRange;

	// Get a handle to the sync var we use for mutex
	HANDLE eventMutex = ::OpenMutex( MUTEX_ALL_ACCESS, true, __WC_MutexName );

	// Establish a sound input channel
	long err = soundPort.StartRecording();
	gQuitting = err != 0;
	
	// Keep doing frames until we get a signal to stop
	while ( ! gQuitting ) {
		
		// Make sure we're not executing anything in the event loop before we start a frame
		if ( ::WaitForSingleObject( eventMutex, INFINITE ) == WAIT_OBJECT_0 ) {

			// Get a freq specrum sample
			binRange = ( (float) gWC -> mStepsPerBin ) / 7.0;
			soundPort.GetSpectrum( gWC -> mNumSampleBins, binRange, gWC -> mSample );

			for ( int i = 0; i < gWC -> mNumSampleBins; i++ ) {
				gWC -> mSample[ i ] = log( .0021 * sqrt( gWC -> mSample[ i ] ) + 2.1 ) - log( 2.1 );
			}
		
			// Record the freq sample in whitecap
			gWC -> RecordSample( EgOSUtils::CurTimeMS(), 26.2 );

			// Update the screen, baby.  Let's shag!
			gWC -> Draw();		
			
			// Release our mutex var
			::ReleaseMutex( eventMutex );
		}
	}
	
	return 0;
}




								
int WINAPI WinMain( HINSTANCE inProcess, 	/*Win32 entry-point routine */
					HINSTANCE, 
					LPSTR, 
					int )
{
	MSG lpMsg;
	DWORD dwThreadID, dwThreadParam = 0;

	gWCModule.hDllInstance = inProcess;
	init( &gWCModule );
	
	// This is what we'll use to prevent GFThread() and WinMain() from both doing GF stuff at once
	HANDLE eventMutex = ::CreateMutex( 0, false, __WC_MutexName );
	
	::SetPriorityClass( inProcess, REALTIME_PRIORITY_CLASS );
	
	HANDLE theWCThread = ::CreateThread( NULL, 0, WCThread, &dwThreadParam, 0, &dwThreadID ); 
	
	::SetThreadPriority( theWCThread, THREAD_PRIORITY_HIGHEST );
	
	while( ::GetMessage( &lpMsg, NULL, 0, 0 ) ) {					/* begin the message loop */
	
		// Make sure we're not executing anything in the event loop before we start a frame
		if ( ::WaitForSingleObject( eventMutex, INFINITE ) == WAIT_OBJECT_0 ) {
		
			// Handle the windoze msg
			TranslateMessage( &lpMsg );
			DispatchMessage( &lpMsg );
			
			// Release our mutex var so WCThread() can execute again
			::ReleaseMutex( eventMutex );
		}
	}
	
	// Signal our main draw thread to exit (and wait), then destroy it
	gQuitting = true;
	while ( ::WaitForSingleObject( theWCThread, INFINITE ) != WAIT_OBJECT_0 ) { }
	::CloseHandle( theWCThread );
	
	// Dispose of our sync var
	::ReleaseMutex( eventMutex );
	
	quit( &gWCModule );
	return( lpMsg.wParam);
}

