// Winamp test visualization library v1.0


#include <windows.h>

#include "WhiteCap.h"
#include "EgOSUtils.h"
#include "RectUtils.h"
#include "CEgFileSpec.h"

#include "Winamp_VIS.H"


char szAppName[] = "WhiteCap"; // Our window class, etc


// returns a winampVisModule when requested. Used in hdr, below
winampVisModule *getModule(int which);

// "member" functions
void config(struct winampVisModule *this_mod); // configuration dialog
int init(struct winampVisModule *this_mod);	   // initialization for module
int Go(struct winampVisModule *this_mod);  // rendering for module 1
void quit(struct winampVisModule *this_mod);   // deinitialization for module

// our window procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);



// Module header, includes version, description, and address of the module retriever function
winampVisHeader hdr = { VIS_HDRVER, "WhiteCap", getModule };



static HWND			gWC_Win = NULL;
WhiteCap*			gWC = NULL;
char				gInitialDir[ 502 ];
long				gInitialDirLen;
	



winampVisModule gWCModule =
{
	"WhiteCap",
	NULL,	// hwndParent
	NULL,	// hDllInstance
	0,		// sRate
	0,		// nCh
	15,		// latencyMS
	2,		// delayMS
	2,		// spectrumNch
	0,		// waveformNch
	{ 0, },	// spectrumData
	{ 0, },	// waveformData
	config,
	init,
	Go, 
	quit
};





// this is the only exported symbol. returns our main header.
// if you are compiling C++, the extern "C" { is necessary, so we just #ifdef it
#ifdef __cplusplus
extern "C" {
#endif
__declspec( dllexport ) winampVisHeader *winampVisGetHeader()
{
	return &hdr;
}
#ifdef __cplusplus
}
#endif

// getmodule routine from the main header. Returns NULL if an invalid module was requested,
// otherwise returns either mod1, mod2 or mod3 depending on 'which'.
winampVisModule *getModule(int inWhich )
{

		
	if ( inWhich == 0 )	
		return &gWCModule;
	else
		return NULL;
}

// configuration. Passed this_mod, as a "this" parameter. Allows you to make one configuration
// function that shares code for all your modules (you don't HAVE to use it though, you can make
// config1(), config2(), etc...)
void config(struct winampVisModule *this_mod)
{
	UtilStr str( WHITECAP_VERS_STR );
	str.Append(  "\n\nRight-click on the WhiteCap window to change 'configs', and use the maximize button to go fullscreen.     \n\nWhiteCap.doc contains info you'll want to know.       ");
	
	MessageBox(this_mod->hwndParent, str.getCStr(),
									"WhiteCap", MB_OK | MB_ICONINFORMATION );
}

// initialization. Registers our window class, creates our window, etc. Again, this one works for
// both modules, but you could make init1() and init2()...
// returns 0 on success, 1 on failure.
int init(struct winampVisModule *this_mod)
{
	
	EgOSUtils::Initialize( this_mod->hDllInstance );

	
	{	// Register our window class
		WNDCLASS wc;
		memset(&wc,0,sizeof(wc));
		wc.lpfnWndProc = WndProc;				// our window procedure
		wc.hInstance = this_mod->hDllInstance;	// hInstance of DLL
		wc.lpszClassName = szAppName;			// our window class name
		wc.hCursor = ::LoadCursor( NULL, IDC_ARROW );
		wc.hbrBackground = NULL;
		wc.style = CS_BYTEALIGNCLIENT;
		
		::RegisterClass(&wc);
	}

	// Make A WhiteCap!
/*	char name[ 800 ];
	::GetModuleFileName( this_mod->hDllInstance, name, 799 );
	UtilStr fname( name );
	fname.Keep( fname.FindLastInstanceOf( '\\' ) );
	fname.Append( "WhiteCap Configs\\" );

	CEgFileSpec folder;
	folder.AssignFolder( fname.getCStr() );
	
*/
	CEgFileSpec folder;
	folder.AssignFolder( "WhiteCap Configs" );


	gWC = new WhiteCap( folder );	

	if ( gWC -> BorderlessWindow() )
		gWC_Win = CreateWindowEx(
			WS_EX_TOPMOST | WS_POPUP,			// these exstyles put a nice small frame, 
												// but also a button in the taskbar
			szAppName,							// our window class name
			this_mod->description,				// use description for a window title
			WS_EX_TOPMOST | WS_POPUP,			// make the window visible with a close button
			100,100,200,200,					// screen position
			NULL,				// parent window (winamp main window)
			NULL,								// no menu
			this_mod->hDllInstance,	0);			// hInstance of DLL
	else
		gWC_Win = CreateWindowEx(
			WS_EX_OVERLAPPEDWINDOW | WS_THICKFRAME,			// these exstyles put a nice small frame, 
												// but also a button in the taskbar
			szAppName,							// our window class name
			this_mod->description,				// use description for a window title
			WS_OVERLAPPEDWINDOW,				// make the window visible with a close button
			100,100,200,200,					// screen position
			NULL,				// parent window (winamp main window)
			NULL,								// no menu
			this_mod->hDllInstance,	0);			// hInstance of DLL
	

	SetWindowLong( gWC_Win,GWL_USERDATA,(LONG)this_mod); // set our user data to a "this" pointer


	gWC -> SetWinPort( gWC_Win );
	::ShowWindow( gWC_Win, SW_SHOWNORMAL );
	gWC -> StoreWinRect();

	return 0;
}

#define SKIP_BINS	14



#define _ABS( x ) (( (x) > 0 ) ? (x) : (-x))
int Go(struct winampVisModule *this_mod)
{
	long sum, idx;
	float*  sample = gWC -> mSample;

/*
	idx = 8;
	for ( int i = 0; i < NUM_SAMPLE_BINS; i++ ) {
		sum = 0;
		for ( int j = 0; j < i+3; j++, idx++ ) 
			sum += ((unsigned) this_mod->spectrumData[0][idx] );
		gSample[ i ] = sum;
	}
	
	idx = 8;
	for ( int i = 0; i < NUM_SAMPLE_BINS; i++ ) {
		sum = 0;
		for ( int j = 0; j < i+3; j++, idx++ ) 
			sum += ((unsigned) this_mod->spectrumData[1][idx] );
		gSample[ i ] = ((float) sum + gSample[ i ]) / (350.0);
	}*/

	int i, j;
	

	float binVal;
	long stepsPerBin = gWC -> mStepsPerBin, numBins = gWC -> mNumSampleBins;
	
	// Restrict case when we have too many WC bins
	if ( numBins > 576 )
		numBins = 576;
	
	// Limit number of bins if there's not enough mp3 player sample to go around
	else if ( numBins * stepsPerBin > (576 - SKIP_BINS) )
		numBins = (576 - SKIP_BINS) / stepsPerBin;


	idx = SKIP_BINS;
	for ( i = 0; i < numBins; i++ ) {
		sum = 0;
		for ( j = 0; j < stepsPerBin; j++, idx++ ) {
			sum += ((unsigned) this_mod->spectrumData[0][idx]);
			sum += ((unsigned) this_mod->spectrumData[1][idx]);
		}
		binVal = ((float) sum);
		sample[ i ] = log( .0026 * sqrt( binVal ) + 1.6 ) - log( 1.6 );
	}
		
	sample[ 0 ] *= 0.30;
	sample[ 1 ] *= 0.75;

	
	// If we didn't have enough bins to fill the WC sample, zero out the rest
	for ( i = numBins; i < gWC -> mNumSampleBins; i++ )
		sample[ i ] = 0;

	gWC -> RecordSample( ::timeGetTime(), 38.0 );

	gWC -> Draw();

	return 0;

}

/*
			sum += ((unsigned) this_mod->spectrumData[1][idx]);
		binVal = ((float) sum + gSample[ i ]) * (0.81 + (float) i / 17) / ( 140 );

		gSample[ i ] = 1.5 * ( log( .008 * binVal + 1.4 ) - log( 1.4 ) );
*/

// cleanup (opposite of init()). Destroys the window, unregisters the window class
void quit(struct winampVisModule *this_mod)
{

	if ( gWC ) {/*
		RECT wr;
		::GetClientRect( hMainWnd, &wr );
		UtilStr str;
		str.Append( wr.top );
		str.Append( "  " );
				str.Append( wr.left );
		str.Append( "  " );
				str.Append( wr.bottom );
		str.Append( "  " );
				str.Append( wr.right );
		str.Append( "  " );
		EgOSUtils::ShowMsg( str );*/
		delete gWC;
		gWC = NULL;
	}
	

	
	::DestroyWindow( gWC_Win ); // delete our window
	::UnregisterClass(szAppName,this_mod->hDllInstance); // unregister window class

	EgOSUtils::Shutdown();
}




/*

// render function for oscilliscope. Returns 0 if successful, 1 if visualization should end.
int render1(struct winampVisModule *this_mod)
{
	int x, y;
	// clear background
	Rectangle(memDC,0,0,288,256);
	// draw oscilliscope
	for (y = 0; y < this_mod->nCh; y ++)
	{
		MoveToEx(memDC,0,(y*256)>>(this_mod->nCh-1),NULL);
		for (x = 0; x < 288; x ++)
		{
			LineTo(memDC,x,(y*256 + this_mod->waveformData[y][x]^128)>>(this_mod->nCh-1));
		}
	}
	{ // copy doublebuffer to window
		HDC hdc = GetDC(hMainWnd);
		BitBlt(hdc,0,0,288,256,memDC,0,0,SRCCOPY);
		ReleaseDC(hMainWnd,hdc);
	}
	return 0;
}

// render function for analyser. Returns 0 if successful, 1 if visualization should end.
int render2(struct winampVisModule *this_mod)
{
	int x, y;
	// clear background
	Rectangle(memDC,0,0,288,256);
	// draw analyser
	for (y = 0; y < this_mod->nCh; y ++)
	{
		for (x = 0; x < 288; x ++)
		{
			MoveToEx(memDC,x,(y*256+256)>>(this_mod->nCh-1),NULL);
			LineTo(memDC,x,(y*256 + 256 - this_mod->spectrumData[y][x])>>(this_mod->nCh-1));
		}
	}
	{ // copy doublebuffer to window
		HDC hdc = GetDC(hMainWnd);
		BitBlt(hdc,0,0,288,256,memDC,0,0,SRCCOPY);
		ReleaseDC(hMainWnd,hdc);
	}
	return 0;
}

// render function for VU meter. Returns 0 if successful, 1 if visualization should end.
int render3(struct winampVisModule *this_mod)
{
	int x, y;
	// clear background
	Rectangle(memDC,0,0,256,32);
	// draw VU meter
	for (y = 0; y < 2; y ++)
	{
		int last=this_mod->waveformData[y][0];
		int total=0;
		for (x = 1; x < 576; x ++)
		{
			total += abs(last - this_mod->waveformData[y][x]);
			last = this_mod->waveformData[y][x];
		}
		total /= 288;
		if (total > 127) total = 127;
		if (y) Rectangle(memDC,128,0,128+total,32);
		else Rectangle(memDC,128-total,0,128,32);
	}
	{ // copy doublebuffer to window
		HDC hdc = GetDC(hMainWnd);
		BitBlt(hdc,0,0,256,32,memDC,0,0,SRCCOPY);
		ReleaseDC(hMainWnd,hdc);
	}
	return 0;
}

*/












// window procedure for our window
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	//static long sLastClickTime = 0;
	MSG msg;

	switch ( message )
	{

			/*
		case WM_NCLMOUSEMOVE:
		case WM_MOUSEMOVED:
			if ( gWC -> IsAtFullScreen() )
				return 0;
			break;
			
		case WM_NCRBUTTONDOWN:
			if ( ! gWC -> IsAtFullScreen() )
				break;*/
		case WM_RBUTTONDOWN:
			if ( (wParam & MK_RBUTTON) && ! gWC -> IsFullscreen() )
				gWC -> SelectConfig();
			return 0;			
			break;
		
		//case WM_NCLBUTTONDOWN:
		
		case WM_LBUTTONDOWN:
			if ( gWC -> IsFullscreen() )
				gWC -> SetFullscreen( false );
			return 0;
			/*
			else if ( message == WM_LBUTTONDOWN ) {
				long curTime = EgOSUtils::CurTimeMS();
				if ( curTime - 300 < sLastClickTime )
					gWC -> SetFullscreen( true );
				else
					sLastClickTime = curTime;
			}
			return 0;	
			break;*/
	
		case WM_CREATE:		return 0;
		case WM_ERASEBKGND:	return 0;
		case WM_PAINT:
			{ // update from doublebuffer
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint( hwnd, &ps );
				gWC -> Refresh();
				gWC -> Draw();
				EndPaint( hwnd, &ps );

			}
			return 0;
		case WM_DESTROY: PostQuitMessage(0); 
			return 0;

		case WM_KEYDOWN:
			if ( ::PeekMessage( &msg, hwnd, WM_CHAR, WM_CHAR, PM_NOREMOVE ) ) {
				if ( gWC -> HandleKey( (unsigned long) (0xFF & msg.wParam) ) )
					return 0; 
				else if ( gWCModule.hwndParent )
					PostMessage( gWCModule.hwndParent,message,wParam,lParam);
			}
		case WM_KEYUP:
			if ( gWCModule.hwndParent ) {
				PostMessage( gWCModule.hwndParent,message,wParam,lParam);
			}
			return 0;
			
		case WM_MOVE:
			return 0;
		case WM_SIZE:
			{
				if ( gWC && hwnd == gWC_Win ) {
					if ( wParam == SIZE_MAXIMIZED ) {
						gWC -> SetFullscreen( true );
						::SetWindowPos( hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE ); }
						
					else if ( ! gWC -> IsFullscreen() ) {
					
						Rect r;
						gWC -> GetWinRect( r );
						gWC -> SetWinPort( hwnd, &r );
					}
				}

			}
			return 0;
		/*	
		case WM_TIMER:
			break;*/
			
	}
	return DefWindowProc(hwnd,message,wParam,lParam);
}

