// Winamp test visualization library v1.0


#include <windows.h>

#include "G-Force.h"
#include "EgOSUtils.h"
#include "RectUtils.h"
#include "CEgFileSpec.h"

#include "vis.h"


char szAppName[] = "G-Force"; // Our window class, etc


// returns a winampVisModule when requested. Used in hdr, below
winampVisModule *getModule(int which);

// "member" functions
void config(struct winampVisModule *this_mod); // configuration dialog
int init(struct winampVisModule *this_mod);	   // initialization for module
int Go(struct winampVisModule *this_mod);  // rendering for module 1
void quit(struct winampVisModule *this_mod);   // deinitialization for module
void CheckForNewSong();

// our window procedure
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);



// Module header, includes version, description, and address of the module retriever function
winampVisHeader hdr = { VIS_HDRVER, "G-Force", getModule };

#define SND_BUF_SIZE 550


static HWND			gGF_Win = NULL;
static GForce*		gGF = NULL;
static UtilStr*		gCurWinTitle = NULL;
static float		gSoundBuf[ SND_BUF_SIZE ];
char				gInitialDir[ 502 ];
long				gInitialDirLen;
long				gLastTitleCheck = -1;



winampVisModule gGFModule =
{
	"G-Force",
	NULL,	// hwndParent
	NULL,	// hDllInstance
	0,		// sRate
	0,		// nCh
	15,		// latencyMS
	2,		// delayMS
	0,		// spectrumNch
	2,		// waveformNch
	{ 0, },	// spectrumData
	{ 0, },	// waveformData
	config,
	init,
	Go, 
	quit
};





// this is the only exported symbol. returns our main header.
// if you are compiling C++, the extern "C" { is necessary, so we just #ifdef it
#ifdef __cplusplus
extern "C" {
#endif
__declspec( dllexport ) winampVisHeader *winampVisGetHeader()
{
	return &hdr;
}
#ifdef __cplusplus
}
#endif

// getmodule routine from the main header. Returns NULL if an invalid module was requested,
// otherwise returns either mod1, mod2 or mod3 depending on 'which'.
winampVisModule *getModule(int inWhich )
{

		
	if ( inWhich == 0 )	
		return &gGFModule;
	else
		return NULL;
}

// configuration. Passed this_mod, as a "this" parameter. Allows you to make one configuration
// function that shares code for all your modules (you don't HAVE to use it though, you can make
// config1(), config2(), etc...)
void config(struct winampVisModule *this_mod)
{
	UtilStr str( GFORCE_VERS_STR );
	str.Append( "\n\nType '?' for help.\nG-Force.doc contains info you'll want to know.       " );
	MessageBox(this_mod->hwndParent,str.getCStr(),
									"G-Force", MB_OK | MB_ICONINFORMATION );
}

// initialization. Registers our window class, creates our window, etc. Again, this one works for
// both modules, but you could make init1() and init2()...
// returns 0 on success, 1 on failure.
int init(struct winampVisModule *this_mod)
{

	
	
	EgOSUtils::Initialize( this_mod->hDllInstance );
	ScreenDevice::sMinDepth = 8;		

	
	{	// Register our window class
		WNDCLASS wc;
		memset(&wc,0,sizeof(wc));
		wc.lpfnWndProc = WndProc;				// our window procedure
		wc.hInstance = this_mod->hDllInstance;	// hInstance of DLL
		wc.lpszClassName = szAppName;			// our window class name
		wc.hCursor = ::LoadCursor( NULL, IDC_ARROW );
		wc.hbrBackground = NULL; //(HBRUSH) ::GetStockObject( BLACK_BRUSH );
		wc.style = CS_BYTEALIGNCLIENT;
		
		::RegisterClass(&wc);
	}




	gGF = new GForce;	
	gCurWinTitle = new UtilStr;
	
	unsigned long wFlags;
	if ( gGF -> BorderlessWindow() ) 
		wFlags = WS_EX_OVERLAPPEDWINDOW | WS_POPUP;
	else
		wFlags = WS_EX_OVERLAPPEDWINDOW | WS_SYSMENU | WS_CAPTION | WS_THICKFRAME | WS_MAXIMIZEBOX | WS_MINIMIZEBOX;
	
	gGF_Win = CreateWindowEx(
		wFlags,								// these exstyles put a nice small frame, 
											// but also a button in the taskbar
		szAppName,							// our window class name
		this_mod->description,				// use description for a window title
		wFlags,								// make the window visible with a close button
		100,100,							// screen position (read from config)
		200,200,							// width & height of window (need to adjust client area later)
		this_mod->hwndParent,				// parent window (winamp main window)
		NULL,								// no menu
		this_mod->hDllInstance,				// hInstance of DLL
		0); // no window creation data


	SetWindowLong(  gGF_Win, GWL_USERDATA,(LONG)this_mod); // set our user data to a "this" pointer

		
	gGF -> SetWinPort(  gGF_Win  );
	::ShowWindow(  gGF_Win , SW_SHOWNORMAL );
	gGF -> StoreWinRect();
	
	return 0;
}


void CheckForNewSong() {
	char title[ 300 ], c;
	long pos;
	
	::GetWindowText( gGFModule.hwndParent, title, 300 );
	
	// If the window text has changed, then parse it for the song info
	if ( gCurWinTitle -> compareTo( title ) ) {
		UtilStr str;
		
		gGF -> mArtist.Wipe();
		gGF -> mAlbum.Wipe();
		
		gCurWinTitle -> Assign( title );
		str.Assign( gCurWinTitle );
		
		// Look for and remove the numerical label winamp may put on ( "XX. " )
		pos = str.contains( ". " );
		c = str.getChar( 1 );
		if ( pos >= 2 && pos <= 4 && c >= '1' &&  c <= '9' )
			str.Trunc( pos + 1, false );
			
		// Look for the winamp tag and remove it
		str.Remove( " - Winamp" );
		
		// A " - " is assumed to separate artist and title
		pos = str.contains( " - " );
		if ( pos ) {
			gGF -> mArtist.Assign( str.getCStr(), pos - 1 );
			gGF -> mSongTitle.Assign( str.getCStr() + pos + 2 ); }
		else
			gGF -> mSongTitle.Assign( str );
			
		// Send the signal
		gGF -> NewSong();
	}
}



#define NUM_BINS 15
#define SKIP_BINS 16
#define STEPS_PER_BIN 1

int Go(struct winampVisModule *this_mod)
{

	long time = EgOSUtils::CurTimeMS();
	if ( time > 1500 + gLastTitleCheck ) {
		CheckForNewSong();
		gLastTitleCheck = time;
	}
	
	long i;
	for ( i = 0; i < 250; i++ ) {
		gSoundBuf[ i ] = (char) (this_mod -> waveformData[0][ 2*i ]);
	}
	

	gGF -> RecordSample( time, gSoundBuf, .0061, 250 );
	
	return 0;

}



// cleanup (opposite of init()). Destroys the window, unregisters the window class
void quit(struct winampVisModule *this_mod)
{

	if ( gGF ) { /*
		RECT wr;
		::GetClientRect( gGF_Win, &wr );
		UtilStr str;
		str.Append( wr.top );
		str.Append( "  " );
				str.Append( wr.left );
		str.Append( "  " );
				str.Append( wr.bottom );
		str.Append( "  " );
				str.Append( wr.right );
		str.Append( "  " );
		EgOSUtils::ShowMsg( str ); */
		delete gGF;		
		gGF = NULL;
		
		delete gCurWinTitle;
		gCurWinTitle = NULL;
	}
	

	
	::DestroyWindow(  gGF_Win  ); // delete our window
	::UnregisterClass(szAppName,this_mod->hDllInstance); // unregister window class

	EgOSUtils::Shutdown();
}




/*

// render function for oscilliscope. Returns 0 if successful, 1 if visualization should end.
int render1(struct winampVisModule *this_mod)
{
	int x, y;
	// clear background
	Rectangle(memDC,0,0,288,256);
	// draw oscilliscope
	for (y = 0; y < this_mod->nCh; y ++)
	{
		MoveToEx(memDC,0,(y*256)>>(this_mod->nCh-1),NULL);
		for (x = 0; x < 288; x ++)
		{
			LineTo(memDC,x,(y*256 + this_mod->waveformData[y][x]^128)>>(this_mod->nCh-1));
		}
	}
	{ // copy doublebuffer to window
		HDC hdc = GetDC(hMainWnd);
		BitBlt(hdc,0,0,288,256,memDC,0,0,SRCCOPY);
		ReleaseDC(hMainWnd,hdc);
	}
	return 0;
}

// render function for analyser. Returns 0 if successful, 1 if visualization should end.
int render2(struct winampVisModule *this_mod)
{
	int x, y;
	// clear background
	Rectangle(memDC,0,0,288,256);
	// draw analyser
	for (y = 0; y < this_mod->nCh; y ++)
	{
		for (x = 0; x < 288; x ++)
		{
			MoveToEx(memDC,x,(y*256+256)>>(this_mod->nCh-1),NULL);
			LineTo(memDC,x,(y*256 + 256 - this_mod->spectrumData[y][x])>>(this_mod->nCh-1));
		}
	}
	{ // copy doublebuffer to window
		HDC hdc = GetDC(hMainWnd);
		BitBlt(hdc,0,0,288,256,memDC,0,0,SRCCOPY);
		ReleaseDC(hMainWnd,hdc);
	}
	return 0;
}

// render function for VU meter. Returns 0 if successful, 1 if visualization should end.
int render3(struct winampVisModule *this_mod)
{
	int x, y;
	// clear background
	Rectangle(memDC,0,0,256,32);
	// draw VU meter
	for (y = 0; y < 2; y ++)
	{
		int last=this_mod->waveformData[y][0];
		int total=0;
		for (x = 1; x < 576; x ++)
		{
			total += abs(last - this_mod->waveformData[y][x]);
			last = this_mod->waveformData[y][x];
		}
		total /= 288;
		if (total > 127) total = 127;
		if (y) Rectangle(memDC,128,0,128+total,32);
		else Rectangle(memDC,128-total,0,128,32);
	}
	{ // copy doublebuffer to window
		HDC hdc = GetDC(hMainWnd);
		BitBlt(hdc,0,0,256,32,memDC,0,0,SRCCOPY);
		ReleaseDC(hMainWnd,hdc);
	}
	return 0;
}

*/






								
int WINAPI WinMain( HINSTANCE, 	/*Win32 entry-point routine */
					HINSTANCE, 
					LPSTR, 
					int )
{
	MSG lpMsg;

	init( &gGFModule );
	
	::SetTimer( gGF_Win, 5556, 10, NULL );

	while( GetMessage( &lpMsg, NULL, 0, 0 ) )					/* begin the message loop */
	{
		Go( &gGFModule );
		TranslateMessage( &lpMsg );
		DispatchMessage( &lpMsg );
	}
	
	::KillTimer( gGF_Win, 5556 );

	
	quit( &gGFModule );
	return( lpMsg.wParam);
}





// window procedure for our window
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static long sLastClickTime = 0;
	MSG msg;

	switch ( message )
	{


			/*
		case WM_NCRBUTTONDOWN:
			if ( ! gWC -> IsAtFullScreen() )
				break;
		case WM_RBUTTONDOWN:
			if ( (wParam & MK_RBUTTON) && ! gGF -> IsFullscreen() ) {
				//gGF -> SelectConfig();
			return 0;			}
			break; */
		
		//case WM_NCLBUTTONDOWN:
		
		case WM_RBUTTONDOWN:
		case WM_LBUTTONDOWN:
			if ( gGF -> IsFullscreen() )
				gGF -> SetFullscreen( false );
			else if ( message == WM_LBUTTONDOWN ) {
				long curTime = EgOSUtils::CurTimeMS();
				if ( curTime - 200 < sLastClickTime )
					gGF -> SetFullscreen( true );
				else
					sLastClickTime = curTime;
			}
			return 0;	
			break;
	
		case WM_CREATE:		return 0;
		case WM_ERASEBKGND:	return 0;
		case WM_PAINT:
			{ // update from doublebuffer
				PAINTSTRUCT ps;
				//if ( ::GetUpdateRect( hwnd, &r false ) ) {
					HDC hdc = ::BeginPaint( hwnd, &ps );
					gGF -> Refresh();
					::EndPaint( hwnd, &ps );
				//}
			}
			return 0;
		case WM_DESTROY: 
			if ( gGF )
				gGF -> StoreWinRect();
			PostQuitMessage(0); 
			return 0;

		case WM_KEYDOWN:
			if ( ::PeekMessage( &msg, hwnd, WM_CHAR, WM_CHAR, PM_NOREMOVE ) ) {
				if ( gGF -> HandleKey( (unsigned long) (0xFF & msg.wParam) ) ) {
					return 0;
				}
			}
		case WM_KEYUP:
			{	// get this_mod from our window's user data
				winampVisModule *this_mod = (winampVisModule *) GetWindowLong(hwnd,GWL_USERDATA);
				PostMessage(this_mod->hwndParent,message,wParam,lParam);
			}
		return 0;
			
		case WM_MOVE:
			return 0;
		case WM_SIZE:
			{
				if ( gGF ) {
					if ( wParam == SIZE_MAXIMIZED )
						gGF -> SetFullscreen( true );
						
					else if ( ! gGF -> IsFullscreen() ) {
					
						Rect r;
						gGF -> GetWinRect( r );
						gGF -> SetWinPort( hwnd, &r );
					}
				}

			}
			return 0;
			
	}
	return DefWindowProc(hwnd,message,wParam,lParam);
}

